import '@angular/localize/init';
import 'moment/locale/de';
import * as i0 from '@angular/core';
import { Injectable, LOCALE_ID, Inject, inject, DestroyRef, EventEmitter, input, model, computed, signal, output, effect, untracked, Output, Input, Component, ViewChild, ElementRef, HostListener, Directive, NgModule } from '@angular/core';
import * as i2$1 from '@sowatech/shared/utilities';
import { ensureArray } from '@sowatech/shared/utilities';
import { Subject } from 'rxjs';
import * as i1 from '@angular/common';
import { CommonModule, DecimalPipe } from '@angular/common';
import * as i1$1 from '@sowatech/shared/shared-config';
import moment from 'moment';
import * as i1$2 from '@sowatech/shared/logger';
import { LoggerService } from '@sowatech/shared/logger';
import { UserPropertiesStorageService } from '@sowatech/shared/user-properties-storage';
import * as i1$3 from '@angular/forms';
import { FormsModule } from '@angular/forms';
import * as i2 from '@sowatech/shared/global-keyboard-shortcuts';
import { GlobalKeyboardShortcutsModule } from '@sowatech/shared/global-keyboard-shortcuts';

/* eslint-disable guard-for-in */
const SELECTION_NOT_VISIBLE_MESSAGE = 'not all selected Ids are visible';
const NO_DATA_MESSAGE = 'no data set';
/**
 * Helfer für AggregateFunctions.
 * Hier muss über die `init` Methode die datasource gesetzt werden
 * und es muss die `getVisibleColumns` methode von außen gesetzt werden
 */
class AggregateFunctionsService {
    constructor() {
        this.subscriptions = [];
        this.values = {};
    }
    init(listDatasource) {
        this.listDatasource = listDatasource;
        this.subscriptions.push(this.listDatasource.onAfterRefresh.subscribe(() => this.recalc()), this.listDatasource.onSelectedIdsChanges.subscribe(() => this.recalc({ refreshIfNotVisible: true })));
    }
    recalc({ refreshIfNotVisible = false } = {}) {
        if (!this.getConfigs)
            return;
        if (this.listDatasource.isExternalRefresh())
            this.tryRecalcWithExternalRefresh({ refreshIfNotVisible });
        else
            this.recalcClientSide();
    }
    cleanUp() {
        this.subscriptions.forEach(s => s.unsubscribe());
    }
    getAggregateItems() {
        if (!this.getConfigs)
            return [];
        const configs = this.getConfigs();
        return configs.flatMap(col => col.aggregateFunctions?.map(af => ({ operator: af.operator, fieldname: col.fieldname, includedData: col.includedAggregateData })) ?? []);
    }
    getAggregateItemsForSelectedData() {
        const allAggregateItems = this.getAggregateItems();
        return allAggregateItems.filter(i => i.includedData === 'selected');
    }
    setByRefreshResult(result) {
        if (!this.getConfigs)
            return;
        if (!result.aggregateResults)
            result.aggregateResults = [];
        if (!result.refreshParams.aggregateParam.aggregatesOnly)
            this.resetValues();
        const configs = result.refreshParams.aggregateParam.aggregatesOnly
            ? this.getConfigsForSelectedData()
            : this.getConfigs();
        for (const col of configs)
            this.values[col.fieldname] = this.getAggregateValuesOfColumn(col, result.aggregateResults);
    }
    getValue(fieldName, operator) {
        return this.values[fieldName]?.find(v => v.operator === operator).value;
    }
    getConfigsForSelectedData() {
        const configs = this.getConfigs();
        return configs.filter(c => c.includedAggregateData === 'selected');
    }
    tryRecalcWithExternalRefresh({ refreshIfNotVisible = false } = {}) {
        try {
            this.recalcWithExternalRefresh();
        }
        catch (error) {
            if (error.message === SELECTION_NOT_VISIBLE_MESSAGE) {
                if (refreshIfNotVisible)
                    this.listDatasource.refreshAggregateFunctionsExternal();
            }
            else if (error.message === NO_DATA_MESSAGE)
                return;
            else
                throw error;
        }
    }
    recalcWithExternalRefresh() {
        if (this.listDatasource.data.length === 0)
            throw new Error(NO_DATA_MESSAGE);
        const selectedIds = this.listDatasource.selectedIds;
        const idField = this.listDatasource.idfield;
        const configs = this.getConfigsForSelectedData().filter(col => col.fieldname && col.aggregateFunctions);
        if (configs.length === 0)
            return;
        const hasOnlyVisibleSelectedIds = selectedIds.every(id => this.listDatasource.data.some(d => d[idField] === id));
        if (!hasOnlyVisibleSelectedIds)
            throw new Error(SELECTION_NOT_VISIBLE_MESSAGE);
        for (const col of configs)
            this.values[col.fieldname] = this.getColumnResult(col);
    }
    recalcClientSide() {
        this.resetValues();
        const configs = this.getConfigs().filter(col => col.fieldname && col.aggregateFunctions);
        if (this.listDatasource.selectedCount > 0 && this.listDatasource.dataBackup.length === 0) {
            // Falls datasource.dataBackup noch nicht gesetzt ist, aber selectedIds aus dem localStorage schon
            return;
        }
        for (const col of configs)
            this.values[col.fieldname] = this.getColumnResult(col);
    }
    resetValues() {
        for (const key in this.values)
            delete this.values[key];
    }
    getColumnResult(col) {
        if (!this.columnHasAggregateFunctions(col))
            return [];
        const values = (col.includedAggregateData ?? 'selected') === 'selected'
            ? this.getValuesOfSelectedItems(col)
            : this.listDatasource.dataFiltered.map(item => this.getTypedValue(item, col.fieldname));
        return col.aggregateFunctions
            .map(agg => ({ operator: agg.operator, value: aggregateFunctionImplementations[agg.operator].calc(values) }));
    }
    getAggregateValuesOfColumn(col, results) {
        if (!this.columnHasAggregateFunctions(col))
            return [];
        return col.aggregateFunctions
            .map(agg => ({ operator: agg.operator, value: results.find(res => res.fieldname === col.fieldname && res.operator === agg.operator)?.value }));
    }
    getValuesOfSelectedItems(col) {
        if (this.listDatasource.selectedCount === 0)
            return [];
        return this.listDatasource.selectedIds.map(id => {
            const item = this.listDatasource.getDataItem(id);
            if (item === undefined)
                throw new Error('Item in Datasource nicht gefunden.'); // Ursache wahrscheinlich Serverseitiges Paging
            return this.getTypedValue(item, col.fieldname);
        });
    }
    getTypedValue(item, fieldname) {
        return item[fieldname];
    }
    columnHasAggregateFunctions(column) {
        return column.aggregateFunctions?.length > 0;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: AggregateFunctionsService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: AggregateFunctionsService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: AggregateFunctionsService, decorators: [{
            type: Injectable
        }] });
const aggregateFunctionImplementations = {
    sum: {
        defaultPrefix: 'Σ ',
        calc: values => values.length > 0 ? values.reduce((a, b) => a + b, 0) : undefined,
    },
    min: {
        defaultPrefix: 'min ',
        calc: values => values.length > 0 ? Math.min(...values) : undefined,
    },
    max: {
        defaultPrefix: 'max ',
        calc: values => values.length > 0 ? Math.max(...values) : undefined,
    },
    count: {
        defaultPrefix: '# ',
        calc: values => values.length,
    },
};

const NEWLINE = '\r\n';
class DatasourceCSVService {
    constructor(datePipe, configService, locale) {
        this.datePipe = datePipe;
        this.configService = configService;
        this.locale = locale;
        this.utf8BOM = new Uint8Array([0xEF, 0xBB, 0xBF]);
    }
    generateAndDownloadCSV(csvData, filename, config) {
        const csvString = this.generateCSV(csvData, config);
        this.executeDownload(csvString, filename);
    }
    generateCSV(csvData, config) {
        const filledConfig = this.applyConfig(csvData, config);
        return this.createCSV(csvData, filledConfig);
    }
    createCSV(csvData, config) {
        let result = '';
        result = this.buildHeaderRow(config);
        for (const row of csvData)
            result += this.buildRow(row, config);
        result = this.removeNewline(result);
        return result;
    }
    removeNewline(lineWithNewline) {
        return lineWithNewline.slice(0, NEWLINE.length * -1);
    }
    removeSeparator(lineWithSeparator) {
        return lineWithSeparator.slice(0, -1);
    }
    buildHeaderRow(config) {
        let headerRow = '';
        for (const col of config.columns)
            headerRow += this.createCell(config.titles === 'columnTitle' ? col.text : col.fieldname, null, config);
        headerRow = this.removeSeparator(headerRow);
        headerRow += NEWLINE;
        return headerRow;
    }
    buildRow(item, config) {
        let row = '';
        config.columns.forEach(column => {
            const content = this.getContent(item, column);
            row += this.createCell(content, column, config);
        });
        this.removeSeparator(row);
        row += NEWLINE;
        return row;
    }
    getContent(item, column) {
        const fieldValue = item[column.fieldname];
        if (column.lookups && column.lookups.length > 0 && this.isSelectItemArray(column.lookups))
            return column.lookups.find(lu => lu.value === fieldValue).text;
        else if (column.type === 'boolean')
            return fieldValue ? column.trueValueText : column.falseValueText;
        return fieldValue;
    }
    isSelectItemArray(lookups) {
        return typeof lookups[0] !== 'string';
    }
    createCell(content, columndata, config) {
        return this.applyQuotes(this.formatCSVField(content, columndata), config.useQuotes) + config.separator;
    }
    applyQuotes(content, useQuotes) {
        const quotes = useQuotes ? '"' : '';
        return quotes + (useQuotes ? content.replace(/"/g, '""') : content) + quotes;
    }
    formatCSVField(content, columndata) {
        if (content === null || content === undefined)
            return '';
        switch (typeof content) {
            case 'number':
                return content.toLocaleString(this.locale, columndata.digitsInfo ? new DigitsInfoHelper(columndata.digitsInfo) : { useGrouping: false, maximumFractionDigits: 20 });
            case 'object':
                return (content instanceof Date) ? this.datePipe.transform(content, columndata.dateformat ?? 'yyyy-MM-dd') : String(content);
            default:
                return String(content);
        }
    }
    applyConfig(csvData, config) {
        const filledConfig = { ...config };
        filledConfig.columns ??= (csvData.length > 0 ? Object.keys(csvData[0]).map(fieldname => ({ fieldname })) : []);
        filledConfig.separator ??= this.configService.csvConfig.separator;
        filledConfig.useQuotes ??= this.configService.csvConfig.useQuotes;
        filledConfig.titles ??= this.configService.csvConfig.titles;
        return filledConfig;
    }
    executeDownload(data, fileName) {
        const blob = new Blob([this.utf8BOM, data], { type: 'text/csv' });
        const tempLink = document.createElement('a');
        tempLink.href = window.URL.createObjectURL(blob);
        tempLink.setAttribute('download', fileName + '.csv');
        tempLink.click();
        window.URL.revokeObjectURL(tempLink.href);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceCSVService, deps: [{ token: i1.DatePipe }, { token: i1$1.DsDatasourceDefaultConfig }, { token: LOCALE_ID }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceCSVService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceCSVService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i1.DatePipe }, { type: i1$1.DsDatasourceDefaultConfig }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [LOCALE_ID]
                }] }] });
class DigitsInfoHelper {
    constructor(digitsInfo) {
        this.minimumIntegerDigits = parseInt(digitsInfo.match(/^(\d+)\./)[1], 10);
        this.minimumFractionDigits = parseInt(digitsInfo.match(/.(\d+)-/)[1], 10);
        this.maximumFractionDigits = parseInt(digitsInfo.match(/-(\d+)$/)[1], 10);
        this.useGrouping = false;
    }
}

class ISortItem {
}
class SortItem {
    constructor(arg1, arg2) {
        this.reverse = false;
        if (typeof (arg1) === 'string') {
            const fieldname = arg1;
            const reverse = arg2 != undefined ? arg2 : false;
            this.fieldname = fieldname;
            this.reverse = reverse;
        }
        else {
            const sortItemData = arg1;
            this.fieldname = sortItemData.fieldname;
            this.reverse = sortItemData.reverse != undefined ? sortItemData.reverse : false;
        }
    }
    static createSortItems(src) {
        const result = new Array();
        if (src) {
            for (const srcItem of src)
                result.push(new SortItem(srcItem));
        }
        return result;
    }
}
//reine daten (zb aus aus json)
class IFilterItem {
}
class FilterItem {
    constructor(arg1, arg2, filterOperator) {
        if (Array.isArray(arg1)) {
            this.fieldnames = arg1;
            const fieldvalues = Array.isArray(arg2) ? arg2 : [arg2];
            this.fieldvalues = [];
            for (const f of fieldvalues) {
                if (typeof (f) !== 'string') {
                    const translation = $localize `:@@Datasource.Filter.StringExpected.msg.Text:fieldvalue string wurde erwartet`;
                    console.error(translation);
                }
                this.fieldvalues.push(f.toLocaleLowerCase());
            }
            this.filterOperator = filterOperator;
        }
        else {
            const filterItemData = arg1;
            this.fieldnames = filterItemData.fieldnames;
            this.fieldvalues = filterItemData.fieldvalues;
            this.filterOperator = filterItemData.filterOperator;
        }
        if (this.filterOperator == undefined)
            this.filterOperator = FilterOperator.Contains;
    }
    fieldnamesAreEqual(arg1) {
        const fieldnames = Array.isArray(arg1) ? arg1 : [arg1];
        return this.fieldnames.length === fieldnames.length &&
            fieldnames.every((f) => this.fieldnames.includes(f));
    }
    get hasFieldValue() {
        return this.fieldvalues && this.fieldvalues.filter(item => item).length > 0;
    }
    valueMatchesFilter(comparer) {
        return this.fieldvalues.some(key => comparer(key));
    }
    static createFilterItems(src) {
        if (!src)
            return [];
        return src.map(srcItem => new FilterItem(srcItem));
    }
    equals(other) {
        return this.fieldnamesAreEqual(other.fieldnames)
            && this.filterOperator === other.filterOperator
            && this.fieldvalues.length === other.fieldvalues.length
            && this.fieldvalues.every((value, index) => value === other.fieldvalues[index]);
    }
    clone() {
        return new FilterItem(this.fieldnames, this.fieldvalues, this.filterOperator);
    }
}
var FilterOperator;
(function (FilterOperator) {
    FilterOperator[FilterOperator["StartsWith"] = 0] = "StartsWith";
    FilterOperator[FilterOperator["IsEqual"] = 1] = "IsEqual";
    FilterOperator[FilterOperator["Contains"] = 2] = "Contains";
    FilterOperator[FilterOperator["IsGreaterOrEqual"] = 3] = "IsGreaterOrEqual";
    FilterOperator[FilterOperator["IsLessOrEqual"] = 4] = "IsLessOrEqual";
})(FilterOperator || (FilterOperator = {}));
class PaginationInfo {
    constructor() {
        this.currentPageIndex = 0;
        this.currentPageNumOfItems = 0;
        this.maxPageIndex = 0;
        this.totalNumOfItems = 0;
        this.pageSize = 0;
    }
}
class AggregateParam {
    constructor() {
        this.items = [];
        /** falls includedData = 'selected' */
        // Wir legen den typ der Id normalerweise nicht fest, aber der Server muss irgendeinen typen erwarten, deshalb string
        this.selectedIds = [];
        /**
         * wenn `true`: Beim Refresh werden nur die Aggregatfunktionen aktualisiert zurück gegeben.
         * Es werden nicht die Daten neu geladen.
         */
        this.aggregatesOnly = false;
    }
}
class RefreshParams {
    constructor() {
        this.paginationInfo = new PaginationInfo();
        this.sortItems = [];
        this.filterItems = [];
        this.aggregateParam = new AggregateParam();
    }
}
class RefreshResult {
    constructor() {
        this.refreshParams = new RefreshParams();
        this.data = [];
        this.aggregateResults = [];
    }
}
/**Refresh Mechanismus, der in der Datasource benutzt wird */
var RefreshType;
(function (RefreshType) {
    /**
     * Der Datasource werden alle Daten übergeben und sie kümmert sich selbst um das Managen der Daten
     */
    RefreshType[RefreshType["internal"] = 0] = "internal";
    /**
     *In der Datasource wird ein Asynchroner Callback ("externalRefresh") gesetzt, der mit RefreshParams ein RefreshResult lädt
     */
    RefreshType[RefreshType["externalRefreshCallback"] = 1] = "externalRefreshCallback";
    /**
     * Das Laden der Daten passiert außerhalb der Datasource.
     * Es wird von einem Output (refreshResultRequested) ausgelöst und das Ergebnis wird per Input (externalRefreshResult) in die Datasource übergeben
     */
    RefreshType[RefreshType["externalRefreshInput"] = 2] = "externalRefreshInput";
})(RefreshType || (RefreshType = {}));

class FilterHelper {
    constructor(loggerService) {
        this.loggerService = loggerService;
        this.items = [];
        this.onFilterItems = new Subject();
        this.filterStrategies = new Array();
        this.filterStrategies.push(new FilterStrategyStartsWith());
        this.filterStrategies.push(new FilterStrategyIsEqual());
        this.filterStrategies.push(new FilterStrategyContains());
        this.filterStrategies.push(new FilterStrategyGreaterOrEqual());
        this.filterStrategies.push(new FilterStrategyLessOrEqual());
    }
    get filterItems() { return this.items; }
    getItemByFieldNames(fieldnames, filterOperator) {
        const resultItems = this.items.filter((item) => item.fieldnamesAreEqual(fieldnames) && item.filterOperator == filterOperator);
        if (resultItems.length > 1)
            console.warn('FilterHandler has duplicate fieldnames');
        return resultItems.length > 0 ? resultItems[0] : null;
    }
    getFirstValueByFieldname(fieldnames, filterOperator) {
        const filterItem = this.getItemByFieldNames(fieldnames, filterOperator);
        return filterItem && filterItem.fieldvalues?.length > 0 ? filterItem.fieldvalues[0] : null;
    }
    clear() {
        this.items = [];
        this.onFilterItems.next(this.items);
    }
    add(arg) {
        // handle overload params
        let filterItems;
        if (Array.isArray(arg))
            filterItems = arg;
        else {
            filterItems = new Array();
            filterItems.push(arg);
        }
        // method body
        for (const filterItem of filterItems) {
            const existingItem = this.getItemByFieldNames(filterItem.fieldnames, filterItem.filterOperator);
            if (existingItem) {
                if (filterItem.hasFieldValue) {
                    existingItem.fieldvalues = filterItem.fieldvalues;
                    existingItem.filterOperator = filterItem.filterOperator;
                }
                else {
                    const idx = this.items.indexOf(existingItem);
                    this.items.splice(idx, 1);
                }
            }
            else {
                if (filterItem.hasFieldValue)
                    this.items.push(filterItem);
            }
        }
        this.onFilterItems.next(this.items);
    }
    set(arg1) {
        this.items = [];
        this.add(arg1);
    }
    execute(dataSourceBackup, dataSource) {
        let workSource = dataSourceBackup.slice(0);
        for (const f of this.items) {
            workSource = workSource.filter(workItem => {
                const filterOperatorStrategies = this.filterStrategies.filter(strat => strat.filterOperator == f.filterOperator);
                const filterOperatorStrategy = filterOperatorStrategies.length > 0 ? filterOperatorStrategies[0] : null;
                if (filterOperatorStrategy == null)
                    this.loggerService.error('No strategy for filteroperator=' + f.filterOperator);
                const filterSuccess = filterOperatorStrategy ? filterOperatorStrategy.filterConditionCompare(workItem, f) : false;
                return filterSuccess;
            });
        }
        dataSource.splice(0);
        dataSource.push(...workSource);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: FilterHelper, deps: [{ token: i1$2.LoggerService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: FilterHelper }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: FilterHelper, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i1$2.LoggerService }] });
class FilterStrategy {
    filterConditionCompare(testedItem, filter) {
        return filter.filterOperator != this.filterOperator || this.filterConditionCompareInternal(testedItem, filter);
    }
    filterConditionCompareInternal(testedItem, filter) {
        let result = false;
        for (const fieldname of filter.fieldnames) {
            const testedValueAsString = this.getValueAsLowerCaseString(testedItem[fieldname]);
            result = this.filterConditionCompareValue(testedValueAsString, filter);
            if (result)
                break;
        }
        return result;
    }
    getValueAsLowerCaseString(value) {
        let result = '';
        if (typeof (value) === 'string')
            result = value;
        if (typeof (value) === 'number')
            result = value.toString();
        if (typeof (value) === 'boolean')
            result = value.toString();
        if (value instanceof Date)
            result = (moment(value)).format('YYYY-MM-DD');
        return result.toLocaleLowerCase();
    }
}
class FilterStrategyStartsWith extends FilterStrategy {
    constructor() {
        super();
        this.filterOperator = FilterOperator.StartsWith;
    }
    filterConditionCompareValue(testedValue, filter) {
        const filterStartsWithWildCard = filter.hasFieldValue && filter.fieldvalues.find(fv => fv.startsWith('*')) != undefined;
        if (filterStartsWithWildCard)
            return this.filterConditionWildcard(testedValue, filter);
        else
            return this.filterConditionStrict(testedValue, filter);
    }
    filterConditionStrict(testedValue, filter) {
        return filter.valueMatchesFilter((inputValue) => testedValue.startsWith(inputValue));
    }
    filterConditionWildcard(testedValue, filter) {
        const filterWithoutWildCard = new FilterItem(filter);
        filterWithoutWildCard.fieldvalues = filterWithoutWildCard.fieldvalues.map(fv => fv.startsWith('*') ? fv.substring(1) : fv);
        return filterWithoutWildCard.valueMatchesFilter((inputValue) => testedValue.includes(inputValue));
    }
}
class FilterStrategyIsEqual extends FilterStrategy {
    constructor() {
        super();
        this.filterOperator = FilterOperator.IsEqual;
    }
    filterConditionCompareValue(testedValue, filter) {
        return filter.valueMatchesFilter((inputValue) => inputValue === testedValue);
    }
}
class FilterStrategyContains extends FilterStrategy {
    constructor() {
        super();
        this.filterOperator = FilterOperator.Contains;
    }
    filterConditionCompareValue(testedValue, filter) {
        return filter.valueMatchesFilter((inputValue) => testedValue.indexOf(inputValue) >= 0);
    }
}
class FilterStrategyGreaterOrEqual extends FilterStrategy {
    constructor() {
        super();
        this.filterOperator = FilterOperator.IsGreaterOrEqual;
    }
    filterConditionCompareValue(testedValue, filter) {
        return filter.valueMatchesFilter((inputValue) => inputValue <= testedValue);
    }
}
class FilterStrategyLessOrEqual extends FilterStrategy {
    constructor() {
        super();
        this.filterOperator = FilterOperator.IsLessOrEqual;
    }
    filterConditionCompareValue(testedValue, filter) {
        return filter.valueMatchesFilter((inputValue) => inputValue >= testedValue);
    }
}

class PaginationHelper {
    constructor(config) {
        this._currentPageIndex = 0;
        this.totalNumOfItems = 0;
        this._maxPageIndex = 0;
        this.pageSize = config.pageSize;
    }
    init(dsPaginationChangedEmitter) {
        this.paginationChangedEmitter = dsPaginationChangedEmitter;
    }
    get paginationInfo() {
        return {
            currentPageIndex: this.currentPageIndex,
            maxPageIndex: this.maxPageIndex,
            totalNumOfItems: this.totalNumOfItems,
            currentPageNumOfItems: this.currentPageIndex < this.maxPageIndex ? this.pageSize : (this.totalNumOfItems - (this.maxPageIndex * this.pageSize)),
            pageSize: this.pageSize
        };
    }
    setPageSize(pageSize, data) {
        this.pageSize = pageSize;
        this.recalcMaxPageIndexAndTotalNum(data);
    }
    get currentPageIndex() {
        return this._currentPageIndex;
    }
    set currentPageIndex(value) {
        //if (value >= 0 && value <= this.maxPageIndex && value != this._currentPageIndex) {
        //    this._currentPageIndex = value;
        //    this.paginationChangedEmitter.emit(this.paginationInfo);
        //}
        if (value >= 0 && value != this._currentPageIndex) {
            this._currentPageIndex = value;
            this.paginationChangedEmitter.next(this.paginationInfo);
        }
    }
    get maxPageIndex() {
        return this._maxPageIndex;
    }
    recalcMaxPageIndexAndTotalNum(data) {
        let haschanges = false;
        if (this.totalNumOfItems !== data.length) {
            this.totalNumOfItems = data.length;
            haschanges = true;
        }
        let newmaxPageIndex = Math.ceil(data.length / this.pageSize) - 1;
        if (newmaxPageIndex < 0)
            newmaxPageIndex = 0;
        if (newmaxPageIndex != this._maxPageIndex) {
            this._maxPageIndex = newmaxPageIndex;
            //if (this.currentPageIndex > this._maxPageIndex) {
            //    this.currentPageIndex = this._maxPageIndex;
            //}
            haschanges = true;
        }
        if (haschanges)
            this.paginationChangedEmitter?.next(this.paginationInfo);
    }
    execute(data) {
        this.recalcMaxPageIndexAndTotalNum(data);
        if (this.pageSize < 1 || data.length <= this.pageSize)
            return;
        else {
            if (this.currentPageIndex < 0)
                this.currentPageIndex = 0;
            //if (this.currentPageIndex > this.maxPageIndex) this.currentPageIndex = this.maxPageIndex;
            const usedPageIndex = this.currentPageIndex <= this.maxPageIndex ? this.currentPageIndex : this.maxPageIndex;
            //let workSource = Enumerable.from(data).skip(this.currentPageIndex * this.pageSize).take(this.pageSize).toArray();
            const workSource = data.slice(usedPageIndex * this.pageSize, usedPageIndex * this.pageSize + this.pageSize);
            data.splice(0); //duplicate ???
            for (const workItem of workSource)
                data.push(workItem);
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: PaginationHelper, deps: [{ token: i1$1.DsDatasourceDefaultConfig }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: PaginationHelper }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: PaginationHelper, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i1$1.DsDatasourceDefaultConfig }] });

const STOREKEY_PREFIX = 'DatasourceRefreshParamsStore.';
const STOREKEY_FILTERITEMS_POSTFIX = '.FilterItems';
const STOREKEY_SORTITEMS_POSTFIX = '.SortItems';
const STOREKEY_PAGEINDEX_POSTFIX = '.PageIndex';
const STOREKEY_PAGESIZE_POSTFIX = '.PageSize';
const STOREKEY_SELECTION_POSTFIX = '.Selection';
class RefreshParamsStoreHelper {
    constructor() {
        this.userProperties = inject(UserPropertiesStorageService);
        this.destroyRef = inject(DestroyRef);
    }
    init(storeKey, onAfterViewInit, onFilterItems, onSortItems, onPaginationChanged, onSelectionChanged, refreshDatasource) {
        this.storeKey = storeKey;
        const subscriptions = [
            onAfterViewInit.subscribe(() => this.loadFromStorage()),
            onFilterItems.subscribe((filterItems) => { this.onDatasourceFilterChanged(filterItems); }),
            onSortItems.subscribe((sortItems) => { this.onDatasourceSortChanged(sortItems); }),
            onPaginationChanged.subscribe((paginationInfo) => { this.onDatasourcePaginationChanged(paginationInfo); }),
            onSelectionChanged.subscribe((selectedIds) => { this.onDatasourceSelectionChanged(selectedIds); }),
        ];
        this.destroyRef.onDestroy(() => subscriptions.forEach(s => s.unsubscribe()));
        this.refreshDatasource = (filterItems, sortItems, pageIndex, selectedIds, pageSize) => { refreshDatasource(filterItems, sortItems, pageIndex, selectedIds, pageSize); };
    }
    clearStorage() {
        this.userProperties.save(this.filterItemsStoreKey, ''); // ? eher delete?
        this.userProperties.save(this.sortItemsStoreKey, '');
        this.userProperties.save(this.paginationStoreKey, '');
        this.userProperties.save(this.selectionStoreKey, '');
        this.userProperties.save(this.pageSizeStoreKey, '');
    }
    loadFromStorage() {
        const storedFilterData = this.userProperties.load(this.filterItemsStoreKey);
        const filterItems = FilterItem.createFilterItems(storedFilterData);
        const storedSortData = this.userProperties.load(this.sortItemsStoreKey);
        const sortItems = SortItem.createSortItems(storedSortData);
        const pageIndex = this.userProperties.load(this.paginationStoreKey);
        const selectedItems = this.userProperties.load(this.selectionStoreKey);
        const pageSize = this.userProperties.load(this.pageSizeStoreKey);
        this.refreshDatasource(filterItems, sortItems, pageIndex, selectedItems, pageSize);
    }
    //---FILTER
    get filterItemsStoreKey() {
        return STOREKEY_PREFIX + this.storeKey + STOREKEY_FILTERITEMS_POSTFIX;
    }
    onDatasourceFilterChanged(filterItems) {
        this.userProperties.save(this.filterItemsStoreKey, filterItems);
    }
    //---SORT
    get sortItemsStoreKey() {
        return STOREKEY_PREFIX + this.storeKey + STOREKEY_SORTITEMS_POSTFIX;
    }
    onDatasourceSortChanged(sortItems) {
        this.userProperties.save(this.sortItemsStoreKey, sortItems);
    }
    //---PAGINATION
    get paginationStoreKey() {
        return STOREKEY_PREFIX + this.storeKey + STOREKEY_PAGEINDEX_POSTFIX;
    }
    onDatasourcePaginationChanged(paginationInfo) {
        this.userProperties.save(this.paginationStoreKey, paginationInfo.currentPageIndex);
        this.userProperties.save(this.pageSizeStoreKey, paginationInfo.pageSize);
    }
    get pageSizeStoreKey() {
        return STOREKEY_PREFIX + this.storeKey + STOREKEY_PAGESIZE_POSTFIX;
    }
    //---SELECTION
    get selectionStoreKey() {
        return STOREKEY_PREFIX + this.storeKey + STOREKEY_SELECTION_POSTFIX;
    }
    onDatasourceSelectionChanged(selectedIds) {
        this.userProperties.save(this.selectionStoreKey, selectedIds);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: RefreshParamsStoreHelper, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: RefreshParamsStoreHelper }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: RefreshParamsStoreHelper, decorators: [{
            type: Injectable
        }] });

class SelectionListHelper {
    constructor(loggerService) {
        this.loggerService = loggerService;
        this.selectedIds = [];
    }
    init(dsSelectedIdsChangedEmitter, dsFocusedIdChangedEventEmitter, dsFocusEventEmitter) {
        this.onSelectedIdsChanged = dsSelectedIdsChangedEmitter;
        this.onFocusedIdChanged = dsFocusedIdChangedEventEmitter;
        this.onFocus = dsFocusEventEmitter;
    }
    doSelectionAndFocusChanged(hasChanged = true) {
        this.doSelectionChanged(hasChanged);
        this.doFocusChanged(hasChanged);
    }
    doSelectionChanged(hasChanged = true) {
        if (hasChanged)
            this.onSelectedIdsChanged.next(this.selectedIds);
    }
    doFocusChanged(hasChanged) {
        const focusedId = this.selectedIds.length > 0 ? this.selectedIds[this.selectedIds.length - 1] : null;
        this.onFocus.next(focusedId);
        if (hasChanged)
            this.onFocusedIdChanged.next(focusedId);
    }
    set(arg1) {
        if (arg1 == null) {
            this.loggerService.warn('SelectionListHelper.set of \'undefined\' not possible');
            return;
        }
        const newSelectedIds = Array.isArray(arg1) ? arg1 : [arg1];
        const hasChanged = this.selectedIds.length !== newSelectedIds.length || this.selectedIds.some((item, index) => item !== newSelectedIds[index]);
        this.selectedIds = newSelectedIds;
        this.doSelectionAndFocusChanged(hasChanged);
    }
    add(arg1) {
        if (arg1 == null) {
            this.loggerService.warn('SelectionListHelper.add of \'undefined\' not possible');
            return;
        }
        let ids;
        if (Array.isArray(arg1))
            ids = arg1;
        else {
            const id = arg1;
            ids = [id];
        }
        const newIds = ids.filter(id => !this.selectedIds.includes(id));
        if (newIds.length > 0) {
            this.selectedIds = this.selectedIds.concat(newIds);
            this.doSelectionChanged();
        }
    }
    remove(id) {
        if (id == null) {
            this.loggerService.warn(`SelectionListHelper.remove of '${id}' not possible`);
            return;
        }
        if (!this.selectedIds.includes(id))
            return;
        this.selectedIds = this.selectedIds.filter(sId => sId != id);
        this.doSelectionChanged();
    }
    clear() {
        if (this.selectedIds.length === 0)
            return;
        this.selectedIds = [];
        this.doSelectionAndFocusChanged();
    }
    contains(id) {
        if (id == null) {
            this.loggerService.warn('call of SelectionListHelper.contains(id) with id==\'undefined\' always returns false');
            return false;
        }
        return this.selectedIds.includes(id);
    }
    getSelectedIds() {
        return this.selectedIds;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: SelectionListHelper, deps: [{ token: i1$2.LoggerService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: SelectionListHelper }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: SelectionListHelper, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: i1$2.LoggerService }] });

class SortHelper {
    constructor() {
        this.fixedItems = [];
        this.items = [];
        this.onSortItems = new EventEmitter();
    }
    get sortItems() { return this.fixedItems.concat(this.items); }
    clear() {
        this.items = [];
        this.emitSortItems();
    }
    add(sortItem) {
        const fixedItem = this.getFixedItemByFieldName(sortItem.fieldname);
        if (fixedItem) {
            fixedItem.reverse = sortItem.reverse;
            this.emitSortItems();
            return;
        }
        const existingItem = this.getItemByFieldName(sortItem.fieldname);
        if (existingItem)
            existingItem.reverse = sortItem.reverse;
        else
            this.items.push(sortItem);
        this.emitSortItems();
    }
    setItem(sortItem) {
        const fixedItem = this.getFixedItemByFieldName(sortItem.fieldname);
        if (fixedItem) {
            fixedItem.reverse = sortItem.reverse;
            this.emitSortItems();
            return;
        }
        this.items = [];
        if (sortItem)
            this.add(sortItem);
    }
    setItems(sortItems) {
        this.items = sortItems;
    }
    /** @returns HasChanges */
    setFixedItems(sortItems) {
        if (!this.fixedItemsChanged(sortItems))
            return false;
        this.fixedItems = sortItems;
        this.emitSortItems();
        return true;
    }
    fixedItemsChanged(sortItems) {
        if (this.fixedItems.length === sortItems.length && this.fixedItems.every(fi => sortItems.find(si => si.fieldname === fi.fieldname)))
            return false;
        return true;
    }
    execute(dataSource) {
        if (this.sortItems?.length > 0)
            dataSource.sort((a, b) => this.sortByMultiple(a, b, 0, this.sortItems.length - 1));
    }
    getItemByFieldName(fieldname) {
        const resultItems = this.items.filter((item) => item.fieldname == fieldname);
        if (resultItems.length > 1)
            console.warn('SortHandler has duplicate fieldnames');
        return resultItems.length > 0 ? resultItems[0] : null;
    }
    getFixedItemByFieldName(fieldname) {
        const resultItems = this.fixedItems.filter((item) => item.fieldname == fieldname);
        if (resultItems.length > 1)
            console.warn('SortHandler has duplicate fieldnames');
        return resultItems.length > 0 ? resultItems[0] : null;
    }
    sortByMultiple(a, b, sortItemIndex = 0, maxSortItemIndex = 0) {
        const sortItem = this.sortItems[sortItemIndex];
        const compVal = this.compare(a[sortItem.fieldname], b[sortItem.fieldname], sortItem.reverse);
        return sortItemIndex >= maxSortItemIndex ? compVal : compVal === 0 ? this.sortByMultiple(a, b, ++sortItemIndex, maxSortItemIndex) : compVal;
    }
    compare(a, b, reverse = false) {
        const reverseFactor = reverse ? -1 : 1;
        if (a == null || b == null) {
            const aNumber = a ? 1 : 0;
            const bNumber = b ? 1 : 0;
            return (aNumber - bNumber) * reverseFactor;
        }
        if (typeof (a) === 'string')
            return a.localeCompare(b) * reverseFactor;
        if (typeof (a) === 'number') {
            if (!a)
                a = 0;
            if (!b)
                b = 0;
            return (a - b) * reverseFactor;
        }
        if (typeof (a) === 'boolean') {
            const aNumber = a ? 1 : 0;
            const bNumber = b ? 1 : 0;
            return (aNumber - bNumber) * reverseFactor;
        }
        if (a instanceof Date) {
            const adate = a ? a : new Date(0);
            const bdate = b ? b : new Date(0);
            return (adate.valueOf() - bdate.valueOf()) * reverseFactor;
        }
        return null;
    }
    emitSortItems() {
        this.onSortItems.emit(this.sortItems);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: SortHelper, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: SortHelper }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: SortHelper, decorators: [{
            type: Injectable
        }] });

/* eslint-disable @angular-eslint/no-output-on-prefix */
/* eslint-disable @angular-eslint/no-input-rename */
class DatasourceComponent {
    constructor(loggerService, selectionList, sortHelper, filterHelper, paginationHelper, refreshParamsStoreHelper, iterableDiffers, keyValueDiffers, config, csvService, aggregateFunctions) {
        this.loggerService = loggerService;
        this.selectionList = selectionList;
        this.sortHelper = sortHelper;
        this.filterHelper = filterHelper;
        this.paginationHelper = paginationHelper;
        this.refreshParamsStoreHelper = refreshParamsStoreHelper;
        this.iterableDiffers = iterableDiffers;
        this.keyValueDiffers = keyValueDiffers;
        this.config = config;
        this.csvService = csvService;
        this.aggregateFunctions = aggregateFunctions;
        this.subscriptions = [];
        this.onAfterViewInit = new Subject();
        /** Liste der Datensätze für die Datasource. Im Gegensatz zum `src` Input, wird das Array nicht direkt bearbeitet */
        this.models = input();
        this.autoSelect = false;
        /**
         * Refresh Mechanismus, der in der Datasource benutzt wird
         * @default internal
         */
        this.refreshType = model(RefreshType.internal);
        /** externalRefreshCallback oder externalRefreshInput */
        this.isExternalRefresh = computed(() => this.refreshType() === RefreshType.externalRefreshCallback || this.refreshType() === RefreshType.externalRefreshInput);
        this.onAfterRefresh = new Subject();
        this.onBeforeRefresh = new Subject();
        this.onAfterInitDatasoure = new Subject();
        this.externalRefreshParams = signal(new RefreshParams());
        this.refreshResultRequested = output();
        this.externalRefreshResult = input(null);
        this.dataBackup = []; //copy of the original provided data. used to refresh
        this.dataChanged = new EventEmitter();
        this.data = []; //the current filtered/sorted data. readonly! use initDataSource(data) to set new data
        this.blockedRefreshRequested = false;
        this.blockCounter = 0;
        this.externalRefreshRunning = false;
        //#region SelectionList
        this.onSelectedIdsChanges = new Subject();
        this.onFocusedIdChanges = new Subject();
        this.onFocus = new Subject();
        this.notFoundSelectedIds = signal([]);
        //#endregion
        this.onPaginationChanged = new Subject();
        this.addRefreshBlocker();
        this.autoSelect = this.config.autoSelect;
        this.idfield = this.config.idfield;
        this.hasChangeTracking = this.config.hasChangeTracking;
        aggregateFunctions.init(this);
        effect(() => this.externalRefreshResultEffectFn());
        effect(() => this.refreshTypeEffectFn());
        effect(() => {
            const models = this.models();
            if (models)
                this.initDataSource([...models]);
        });
    }
    ngOnInit() {
        this.initPaginationHelper();
        this.initSelectionList();
        //this.initFilterHelper();//todo: eventemitter in ds component verlagern wie beoi paginationHelper
        //this.initSortHelper();//todo
        //this.initGroupHelper();//todo
        this.subscriptions.push(this.onSelectedIdsChanges.subscribe(ids => this.checkSelectedIdsCanBeFound(ids)));
        this.initRefreshParamsStoreHelper();
    }
    ngAfterViewInit() {
        this.onAfterViewInit.next();
        //bei externalRefreshInput wird durch einen effect nach dem ngAfterViewInit ein refresh Aufgerufen
        const catchUpRefresh = this.refreshType() !== RefreshType.externalRefreshInput;
        this.removeRefreshBlocker(catchUpRefresh);
    }
    ngDoCheck() {
        if (!this.hasChangeTracking)
            return;
        //-- liste selbst
        const dataBackupIterableChanges = this.dataBackupIterableDiffer && this.dataBackupIterableDiffer.diff(this.dataBackup);
        //-- jedes dataItem der liste
        const allDataItemChangesDictionary = {};
        let hasDataItemChanges = false;
        for (const dataItem of this.dataBackup) {
            const dataItemDiffer = this.dataItemDifferDictionary[this.getDataItemId(dataItem)];
            const dataItemChanges = dataItemDiffer && dataItemDiffer.diff(dataItem);
            if (dataItemChanges) {
                allDataItemChangesDictionary[this.getDataItemId(dataItem)] = dataItemChanges;
                hasDataItemChanges = true;
            }
        }
        if (dataBackupIterableChanges || hasDataItemChanges)
            this.dataBackupChanged(dataBackupIterableChanges, allDataItemChangesDictionary);
    }
    ngOnDestroy() {
        this.aggregateFunctions.cleanUp();
        this.subscriptions.forEach(s => s.unsubscribe());
    }
    set sourceData(data) {
        this.initDataSource(data);
    }
    set _changeTracking(value) {
        this.hasChangeTracking = value;
    }
    set doSetPageSize(value) {
        this.setPageSize(value);
    }
    set sortBy(fieldname) {
        this.setSort(fieldname);
    }
    refreshTypeEffectFn() {
        if (this.refreshType() === RefreshType.externalRefreshInput) {
            while (this.refreshIsBlocked)
                this.removeRefreshBlocker(false);
            this.refresh();
        }
    }
    set externalRefresh(method) {
        this._externalRefresh = method;
        if (method) {
            this.refreshType.set(RefreshType.externalRefreshCallback);
            this.externalRefreshParams.set(new RefreshParams());
            this.refresh();
        }
        else {
            this.externalRefreshParams.set(null);
            this.refreshType.set(RefreshType.internal);
        }
    }
    get externalRefresh() {
        return this._externalRefresh;
    }
    externalRefreshResultEffectFn() {
        if (this.externalRefreshResult())
            untracked(() => this.handleExternalRefreshResult(this.externalRefreshResult()));
    }
    initDataBackupDiffer() {
        // see https://www.concretepage.com/questions/667
        if (!this.hasChangeTracking)
            return;
        //-- liste selbst
        this.dataBackupIterableDiffer = this.iterableDiffers.find(this.dataBackup).create(); //hat scheinar keinen effekt hier this.dataBackup zu übergeben; wirkt sich nciuht wie erwartet als initialiserung der changes aus
        this.dataBackupIterableDiffer.diff(this.dataBackup); //initialChanges sind dirty durch das initiale erzeugen; nächster aufruf von diff jat dann keine changes mehr.
        //-- jedes dataItem der liste
        this.dataItemDifferDictionary = {};
        for (const dataItem of this.dataBackup) {
            const dataItemDiffer = this.keyValueDiffers.find(dataItem).create();
            this.dataItemDifferDictionary[this.getDataItemId(dataItem)] = dataItemDiffer;
            dataItemDiffer.diff(dataItem); //initialChanges sind dirty durch das initiale erzeugen; nächster aufruf von diff jat dann keine changes mehr.
        }
    }
    getDataItemId(dataItem) {
        return dataItem[this.idfield];
    }
    dataBackupChanged(iterableChanges, dataItemChanges) {
        const dataItemChangesHasAny = Object.keys(dataItemChanges).length > 0;
        if (dataItemChangesHasAny)
            this.dataChanged.emit({ dataItemChanges });
        /* If you want to see details then use
          kvchanges.forEachChangedItem((record) => ...);
        */
    }
    get dataFiltered() {
        //the filtered data (ignoring pagination)
        const filteredData = [];
        this.doInternalRefresh(filteredData, false);
        return filteredData;
    }
    get hasData() {
        return this.dataBackup.length > 0;
    }
    getDataItem(id) {
        return this.dataBackup.find(item => item[this.idfield] == id);
    }
    getDataDistinct(fieldname) {
        return Array.from(new Set(this.dataBackup.map(d => d[fieldname]))); //.sort(); evtl. noch sorten
    }
    focusNextOrFirstRow() {
        if (!this.data || !this.idfield)
            return;
        if (this.focusedItem) {
            const currentData = this.data.find(d => d[this.idfield] == this.focusedItem[this.idfield]);
            const index = this.data.indexOf(currentData);
            if (this.data[index + 1] && this.data[index + 1][this.idfield])
                this.focus(this.data[index + 1][this.idfield]);
        }
        else
            this.focus((this.data[0])[this.idfield], true);
    }
    focusPreviousRow() {
        if (!this.data || !this.idfield)
            return;
        if (this.focusedItem) {
            // eslint-disable-next-line eqeqeq
            const currentData = this.data.find(d => d[this.idfield] == this.focusedItem[this.idfield]);
            const index = this.data.indexOf(currentData);
            if (this.data[index - 1] && this.data[index - 1][this.idfield])
                this.focus(this.data[index - 1][this.idfield]);
        }
    }
    async initDataSource(data) {
        if (!data)
            return;
        this.dataBackup = data.slice(0);
        this.data = data;
        if (data.length > 0)
            this.autoDetectIdField(data[0]);
        this.initDataBackupDiffer();
        await this.refresh();
        this.autoSelectFirstItem();
        this.checkSelectedIdsCanBeFound(this.selectedIds);
        this.onAfterInitDatasoure.next(null);
    }
    autoSelectFirstItem() {
        if (!this.autoSelect)
            return;
        if (!this.focusedItem && this.data.length > 0) {
            const firstModel = this.data[0];
            this.focus(firstModel[this.idfield], true);
        }
    }
    autoDetectIdField(dataItem) {
        if (this.idfield)
            return;
        if (dataItem.id !== undefined)
            this.idfield = 'id';
        else if (dataItem.Id !== undefined)
            this.idfield = 'Id';
        else if (dataItem.ID !== undefined)
            this.idfield = 'ID';
        else
            this.loggerService.warn('datasource.idfield is not specified');
    }
    get refreshIsBlocked() {
        return this.blockCounter > 0;
    }
    addRefreshBlocker() {
        this.blockCounter++;
    }
    removeRefreshBlocker(catchUpRefresh = true) {
        this.blockCounter--;
        if (this.blockCounter < 0)
            this.blockCounter = 0;
        if (this.blockCounter === 0) {
            if (catchUpRefresh)
                this.catchUpRefresh();
            else
                this.blockedRefreshRequested = false;
        }
    }
    catchUpRefresh() {
        if (this.blockedRefreshRequested) {
            this.refresh();
            this.blockedRefreshRequested = false;
        }
    }
    async refresh() {
        await untracked(async () => {
            if (this.refreshIsBlocked) {
                this.blockedRefreshRequested = true;
                return;
            }
            this.onBeforeRefresh.next();
            switch (this.refreshType()) {
                case RefreshType.internal:
                    this.doInternalRefresh(this.data);
                    this.afterRefresh();
                    break;
                case RefreshType.externalRefreshCallback:
                    await this.doExternalRefresh();
                    this.afterRefresh();
                    break;
                case RefreshType.externalRefreshInput:
                    this.requestNewRefreshResult();
                    break;
            }
        });
    }
    afterRefresh() {
        if (this.focusAfterRefreshId != null) {
            const focusId = this.focusAfterRefreshId;
            this.focusAfterRefreshId = null;
            this.focus(focusId, true);
        }
        this.onAfterRefresh.next();
    }
    async refreshAggregateFunctionsExternal() {
        if (this.refreshType() === RefreshType.externalRefreshCallback) {
            const refreshParams = new RefreshParams();
            refreshParams.filterItems = this.filterHelper.filterItems;
            refreshParams.aggregateParam.items = this.aggregateFunctions.getAggregateItems();
            refreshParams.aggregateParam.selectedIds = this.selectedIds.map(id => id.toString());
            refreshParams.aggregateParam.aggregatesOnly = true;
            refreshParams.idField = this.idfield;
            const result = await this.externalRefresh(refreshParams);
            this.aggregateFunctions.setByRefreshResult(result);
        }
        else if (this.refreshType() === RefreshType.externalRefreshInput) {
            this.updateExternalRefreshParams(refreshParams => {
                refreshParams.filterItems = this.filterHelper.filterItems;
                refreshParams.aggregateParam.items = this.aggregateFunctions.getAggregateItems();
                refreshParams.aggregateParam.selectedIds = this.selectedIds.map(id => id.toString());
                refreshParams.aggregateParam.aggregatesOnly = true;
                refreshParams.idField = this.idfield;
            });
            this.emitRefreshResultRequested(this.externalRefreshParams());
        }
    }
    doInternalRefresh(destData, doPagination = true) {
        this.filterHelper.execute(this.dataBackup, destData);
        this.sortHelper.execute(destData);
        if (doPagination)
            this.paginationHelper.execute(destData);
    }
    getExternalRefreshParams() {
        return this.externalRefreshParams();
    }
    async doExternalRefresh() {
        this.setExternalRefreshParams();
        if (this.externalRefreshRunning) {
            this.nextExternalRefresh = () => this.externalRefresh(this.externalRefreshParams());
            return;
        }
        try {
            this.externalRefreshRunning = true;
            const result = await this.getNextExternalRefreshResult();
            this.externalRefreshRunning = false;
            this.handleRefreshResult(result);
        }
        catch (error) {
            this.loggerService.error(error);
        }
    }
    requestNewRefreshResult() {
        this.setExternalRefreshParams();
        this.emitRefreshResultRequested(this.externalRefreshParams());
    }
    emitRefreshResultRequested(refreshParams) {
        const deepCopy = this.deepCopy(refreshParams);
        this.refreshResultRequested.emit(deepCopy);
    }
    setExternalRefreshParams() {
        this.externalRefreshParams.update(refreshParams => {
            refreshParams.paginationInfo.pageSize = this.paginationHelper.pageSize; //andere pageInfos koennen nicht durch paginationHelper ermittelt werden
            refreshParams.sortItems = this.sortHelper.sortItems;
            refreshParams.filterItems = this.filterHelper.filterItems;
            refreshParams.aggregateParam.items = this.aggregateFunctions.getAggregateItems();
            refreshParams.aggregateParam.selectedIds = this.selectedIds.map(id => id.toString());
            refreshParams.idField = this.idfield;
            return refreshParams;
        });
    }
    /**Für Inline Updates der externalRefreshParams */
    updateExternalRefreshParams(updateFn) {
        this.externalRefreshParams.update(refreshParams => {
            updateFn(refreshParams);
            return refreshParams;
        });
    }
    handleRefreshResult(result) {
        this.data.splice(0);
        this.dataBackup = [];
        result.refreshParams.aggregateParam ??= new AggregateParam(); // fix SHANG-145
        this.externalRefreshParams.set(result.refreshParams);
        this.onPaginationChanged.next(result.refreshParams.paginationInfo);
        if (result.data) {
            this.pushModelsToArray(result.data, this.data);
            this.dataBackup = this.data.slice();
            if (!this.idfield && this.data.length > 0)
                this.autoDetectIdField(this.data[0]);
        }
        this.aggregateFunctions.setByRefreshResult(result);
    }
    handleExternalRefreshResult(result) {
        const deepCopy = this.deepCopy(result);
        this.handleRefreshResult(deepCopy);
        this.updateExternalRefreshParams(refreshParams => refreshParams.paginationInfo.goToPageOfId = undefined);
        this.afterRefresh();
    }
    pushModelsToArray(incomingData, targetArray) {
        if (this.externalDataItem2ViewModel)
            targetArray.push(...incomingData.map(dataItem => this.externalDataItem2ViewModel(dataItem)));
        else
            targetArray.push(...incomingData);
    }
    async getNextExternalRefreshResult() {
        let result = await this.externalRefresh(this.externalRefreshParams());
        while (this.nextExternalRefresh) {
            const nextRefresh = this.nextExternalRefresh;
            this.nextExternalRefresh = undefined;
            result = await nextRefresh(this.externalRefreshParams());
        }
        return result;
    }
    getPageIndexById(id) {
        if (this.isExternalRefresh()) {
            this.loggerService.warn('getPageIndexById not implemented for externalRefresh');
            return -1;
        }
        else {
            const searchData = [];
            this.doInternalRefresh(searchData, false); //normales refresh ausfuehren in eine such-liste, OHNE die pagination auszufuehren
            const itemIdx = searchData.indexOf(searchData.find(d => d[this.idfield] == id));
            let pageIndex = -1;
            if (this.paginationHelper.pageSize > 0)
                pageIndex = Math.ceil((itemIdx + 1) / this.paginationHelper.pageSize) - 1;
            return pageIndex;
        }
    }
    //#region CSV Download
    async downloadDataAsCSV(fileName, config) {
        const csvData = await this.getCSVData();
        const name = this.assembleCSVName(fileName);
        this.csvService.generateAndDownloadCSV(csvData, name, config);
    }
    assembleCSVName(fileName) {
        const date = new Date();
        return `${fileName ?? this.storeKey ?? 'downloadDataAsCSV'}_${date.getFullYear()}-${date.getMonth() + 1}-${date.getDate()}`;
    }
    async getCSVData() {
        switch (this.refreshType()) {
            case RefreshType.internal: {
                const csvData = [];
                this.doInternalRefresh(csvData, false);
                return csvData;
            }
            case RefreshType.externalRefreshCallback:
                return await this.externalCSVDataRefresh();
            case RefreshType.externalRefreshInput:
                throw new Error('CSV Export for ExternalRefreshInput not implemented');
        }
    }
    async externalCSVDataRefresh() {
        const refreshParams = new RefreshParams();
        refreshParams.paginationInfo.pageSize = 999999;
        refreshParams.sortItems = this.sortHelper.sortItems;
        refreshParams.filterItems = this.filterHelper.filterItems;
        try {
            const refreshResult = await this.externalRefresh(refreshParams);
            const result = [];
            if (refreshResult.data)
                this.pushModelsToArray(refreshResult.data, result);
            return result;
        }
        catch (error) {
            this.loggerService.error(error);
        }
    }
    //#endregion
    setSort(field) {
        this.sortHelper.setItem(new SortItem(field));
        this.refresh();
    }
    get onSortItems() {
        return this.sortHelper.onSortItems;
    }
    getSortItems() {
        return this.sortHelper.sortItems;
    }
    sortAsc(fieldname, addSorting = false) {
        addSorting ? this.sortHelper.add(new SortItem(fieldname)) : this.sortHelper.setItem(new SortItem(fieldname));
        this.refresh();
    }
    sortDesc(fieldname, addSorting = false) {
        addSorting ? this.sortHelper.add(new SortItem(fieldname, true)) : this.sortHelper.setItem(new SortItem(fieldname, true));
        this.refresh();
    }
    sort(sortItems) {
        if (sortItems instanceof Array)
            this.sortHelper.setItems(sortItems);
        else
            this.sortHelper.setItem(sortItems);
        this.refresh();
    }
    clearSort() {
        this.sortHelper.clear();
        this.refresh();
    }
    setFixedSort(fieldnames) {
        const items = fieldnames.map(fieldname => new SortItem(fieldname));
        const hasChanges = this.sortHelper.setFixedItems(items);
        if (hasChanges)
            this.refresh();
    }
    //----------------
    get onFilterItems() {
        return this.filterHelper.onFilterItems;
    }
    setFilter(arg1, arg2, filteroperator) {
        if (arg2 === undefined) {
            const filterItems = arg1;
            this.filterHelper.set(filterItems);
        }
        else {
            const fieldnames = Array.isArray(arg1) ? arg1 : [arg1];
            const fieldvalues = Array.isArray(arg2) ? arg2 : [arg2];
            this.filterHelper.set(new FilterItem(fieldnames, fieldvalues, filteroperator));
        }
        this.refresh();
    }
    addFilter(arg1, arg2, filteroperator) {
        const filterItems = arg2 === undefined
            ? arg1
            : [new FilterItem(ensureArray(arg1), ensureArray(arg2), filteroperator)];
        const oldItems = this.filterHelper.filterItems.map(fi => fi.clone());
        this.filterHelper.add(filterItems);
        const newItems = this.filterHelper.filterItems.map(fi => fi.clone());
        const filterChanged = !this.filterItemsEqual(oldItems, newItems);
        if (filterChanged)
            this.refresh();
    }
    filterItemsEqual(oldItems, newItems) {
        return oldItems.length === newItems.length && oldItems.every((oldItem, i) => oldItem.equals(newItems[i]));
    }
    clearFilter() {
        this.filterHelper.clear();
        this.refresh();
    }
    getFilterItems() {
        return this.filterHelper.filterItems;
    }
    getFirstFilterValueByFieldname(fieldnames, filterOperator) {
        return this.filterHelper.getFirstValueByFieldname(fieldnames, filterOperator);
    }
    initSelectionList() {
        this.selectionList.init(this.onSelectedIdsChanges, this.onFocusedIdChanges, this.onFocus);
    }
    checkSelectedIdsCanBeFound(selectedIds) {
        if (this.isExternalRefresh())
            return;
        const notFoundIds = [];
        for (const selectedId of selectedIds) {
            const model = this.dataBackup.find(data => data[this.idfield] === selectedId);
            if (!model)
                notFoundIds.push(selectedId);
        }
        if (notFoundIds.length > 0)
            this.loggerService.warn('Cannot find following Ids in datasource:' + notFoundIds);
        this.notFoundSelectedIds.set(notFoundIds);
    }
    get selectedCount() {
        return this.selectedIds.length;
    }
    get selectedIds() {
        return this.selectionList.getSelectedIds();
    }
    isSelected(id) {
        return this.selectionList.contains(id);
    }
    focusAfterRefresh(id) {
        this.focusAfterRefreshId = id;
    }
    async focus(id, gotoPage = false) {
        if (!gotoPage) {
            this.selectionList.set(id);
            return;
        }
        if (!this.isExternalRefresh()) {
            const pageIndex = this.getPageIndexById(id);
            if (pageIndex >= 0)
                this.gotoPage(pageIndex);
            else
                this.loggerService.warn(`cannot go to page of item  id=${id}. pageindex=${pageIndex}`);
        }
        else if (!this.data.some(d => d[this.idfield] === id)) {
            this.updateExternalRefreshParams(refreshParams => refreshParams.paginationInfo.goToPageOfId = id.toString());
            await this.refresh();
            if (this.refreshType() !== RefreshType.externalRefreshInput)
                this.updateExternalRefreshParams(refreshParams => refreshParams.paginationInfo.goToPageOfId = undefined);
        }
        this.selectionList.set(id);
    }
    get focusedId() {
        return this.selectedCount > 0
            ? this.selectedIds[this.selectedCount - 1]
            : null;
    }
    get focusedItem() {
        let focusedItem = null;
        if (this.focusedId != null) {
            const focusedItems = this.dataBackup.filter(item => item[this.idfield] == this.focusedId);
            if (focusedItems.length > 0)
                focusedItem = focusedItems[0];
        }
        return focusedItem;
    }
    setSelection(arg1) {
        this.selectionList.set(arg1);
    }
    addSelection(arg1) {
        this.selectionList.add(arg1);
    }
    removeSelection(id) {
        this.selectionList.remove(id);
    }
    toggleSelection(id) {
        if (this.isSelected(id))
            this.selectionList.remove(id);
        else
            this.selectionList.add(id);
    }
    clearSelection() {
        this.selectionList.clear();
    }
    selectAll() {
        this.setSelection(this.dataFiltered.map(d => d[this.idfield]));
    }
    initPaginationHelper() {
        this.paginationHelper.init(this.onPaginationChanged);
    }
    setPageSize(pageSize) {
        if (this.paginationHelper.pageSize === pageSize)
            return;
        this.paginationHelper.setPageSize(pageSize, this.data);
        this.onPaginationChanged.next(this.paginationHelper.paginationInfo);
        this.refresh();
    }
    get paginationInfo() {
        if (this.isExternalRefresh())
            return this.externalRefreshParams().paginationInfo;
        else
            return this.paginationHelper.paginationInfo;
    }
    gotoPage(pageIndex) {
        if (this.isExternalRefresh())
            this.updateExternalRefreshParams(refreshParams => refreshParams.paginationInfo.currentPageIndex = pageIndex);
        else
            this.paginationHelper.currentPageIndex = pageIndex;
        this.refresh();
    }
    async gotoPageOfFocusedItem() {
        if (this.isExternalRefresh()) {
            if (this.focusedItem)
                return;
            this.updateExternalRefreshParams(refreshParams => refreshParams.paginationInfo.goToPageOfId = this.focusedId.toString());
            await this.refresh();
            if (this.refreshType() !== RefreshType.externalRefreshInput)
                this.updateExternalRefreshParams(refreshParams => refreshParams.paginationInfo.goToPageOfId = undefined);
        }
        else {
            const pageIndex = this.getPageIndexById(this.focusedId);
            if (pageIndex >= 0)
                this.gotoPage(pageIndex);
        }
    }
    get currentPageIndex() {
        if (this.isExternalRefresh())
            return this.externalRefreshParams().paginationInfo.currentPageIndex;
        else
            return this.paginationHelper.currentPageIndex;
    }
    get maxPageIndex() {
        if (this.isExternalRefresh())
            return this.externalRefreshParams().paginationInfo.maxPageIndex;
        else
            return this.paginationHelper.maxPageIndex;
    }
    toggleAllFilteredSelected(selectAllFiltered) {
        if (selectAllFiltered)
            this.setSelection(this.dataFiltered.map(d => d[this.idfield]));
        else
            this.clearSelection();
    }
    //---RefreshParamStoreHelper
    initRefreshParamsStoreHelper() {
        if (!this.storeKey)
            return;
        this.refreshParamsStoreHelper.init(this.storeKey, this.onAfterViewInit, this.onFilterItems, this.onSortItems, this.onPaginationChanged, this.onSelectedIdsChanges, (filterItems, sortItems, pageIndex, selectedIds, pageSize) => this.refreshParamsStore(filterItems, sortItems, pageIndex, selectedIds, pageSize));
    }
    refreshParamsStore(filterItems, sortItems, pageIndex, selectedIds, pageSize) {
        this.addRefreshBlocker();
        this.setFilter(filterItems);
        if (sortItems)
            this.sort(sortItems);
        if (pageIndex != undefined)
            this.gotoPage(pageIndex);
        if (pageSize)
            this.setPageSize(pageSize);
        if (selectedIds)
            this.setSelection(selectedIds);
        this.removeRefreshBlocker();
    }
    deepCopy(object) {
        return JSON.parse(JSON.stringify(object));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceComponent, deps: [{ token: i1$2.LoggerService }, { token: SelectionListHelper }, { token: SortHelper }, { token: FilterHelper }, { token: PaginationHelper }, { token: RefreshParamsStoreHelper }, { token: i0.IterableDiffers }, { token: i0.KeyValueDiffers }, { token: i1$1.DsDatasourceDefaultConfig }, { token: DatasourceCSVService }, { token: AggregateFunctionsService }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.2.13", type: DatasourceComponent, isStandalone: false, selector: "ds-datasource", inputs: { sourceData: { classPropertyName: "sourceData", publicName: "src", isSignal: false, isRequired: false, transformFunction: null }, models: { classPropertyName: "models", publicName: "models", isSignal: true, isRequired: false, transformFunction: null }, _changeTracking: { classPropertyName: "_changeTracking", publicName: "changeTracking", isSignal: false, isRequired: false, transformFunction: null }, doSetPageSize: { classPropertyName: "doSetPageSize", publicName: "pageSize", isSignal: false, isRequired: false, transformFunction: null }, storeKey: { classPropertyName: "storeKey", publicName: "store-key", isSignal: false, isRequired: false, transformFunction: null }, idfield: { classPropertyName: "idfield", publicName: "idfield", isSignal: false, isRequired: false, transformFunction: null }, sortBy: { classPropertyName: "sortBy", publicName: "sortBy", isSignal: false, isRequired: false, transformFunction: null }, autoSelect: { classPropertyName: "autoSelect", publicName: "autoSelect", isSignal: false, isRequired: false, transformFunction: null }, refreshType: { classPropertyName: "refreshType", publicName: "refreshType", isSignal: true, isRequired: false, transformFunction: null }, externalRefreshResult: { classPropertyName: "externalRefreshResult", publicName: "externalRefreshResult", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { refreshType: "refreshTypeChange", onAfterRefresh: "onAfterRefresh", onBeforeRefresh: "onBeforeRefresh", onAfterInitDatasoure: "onAfterInitDatasoure", refreshResultRequested: "refreshResultRequested" }, providers: [
            SelectionListHelper,
            SortHelper,
            FilterHelper,
            PaginationHelper,
            RefreshParamsStoreHelper,
            DatasourceCSVService,
            AggregateFunctionsService,
        ], ngImport: i0, template: '', isInline: true }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'ds-datasource',
                    template: '',
                    providers: [
                        SelectionListHelper,
                        SortHelper,
                        FilterHelper,
                        PaginationHelper,
                        RefreshParamsStoreHelper,
                        DatasourceCSVService,
                        AggregateFunctionsService,
                    ],
                    standalone: false
                }]
        }], ctorParameters: () => [{ type: i1$2.LoggerService }, { type: SelectionListHelper }, { type: SortHelper }, { type: FilterHelper }, { type: PaginationHelper }, { type: RefreshParamsStoreHelper }, { type: i0.IterableDiffers }, { type: i0.KeyValueDiffers }, { type: i1$1.DsDatasourceDefaultConfig }, { type: DatasourceCSVService }, { type: AggregateFunctionsService }], propDecorators: { sourceData: [{
                type: Input,
                args: ['src']
            }], _changeTracking: [{
                type: Input,
                args: ['changeTracking']
            }], doSetPageSize: [{
                type: Input,
                args: ['pageSize']
            }], storeKey: [{
                type: Input,
                args: ['store-key']
            }], idfield: [{
                type: Input,
                args: ['idfield']
            }], sortBy: [{
                type: Input
            }], autoSelect: [{
                type: Input
            }], onAfterRefresh: [{
                type: Output
            }], onBeforeRefresh: [{
                type: Output
            }], onAfterInitDatasoure: [{
                type: Output
            }] } });
// erfordert changeTracking:true
class DataChangeEvent {
}

class DatasourceSelectionCheckboxComponent {
    constructor() {
        this.subscriptions = new Array();
        //isChecked: boolean = false;
        this._isChecked = false;
        this.multiSelectIsChecked = false;
    }
    ngOnInit() {
        this.subscriptions.push(this.datasource.onSelectedIdsChanges.subscribe((selectedIds) => {
            this.onSelectedIdsChanges(selectedIds);
        }), this.datasource.onAfterRefresh.subscribe(() => {
            this.toogleIsCheckedIfNesseary();
        }));
        this._isChecked = this.selectId !== undefined && this.datasource.isSelected(this.selectId);
    }
    ngOnDestroy() {
        this.subscriptions.map(s => s.unsubscribe());
    }
    toogleIsCheckedIfNesseary() {
        if (!this.selectId && this.datasource.selectedIds.length == 0)
            this.multiSelectIsChecked = false;
    }
    set isChecked(value) {
        if (this.selectId) {
            if (value == this._isChecked)
                return;
            if (value) {
                if (this.allowMultiSelect)
                    this.datasource.addSelection(this.selectId);
                else
                    this.datasource.focus(this.selectId);
            }
            else
                this.datasource.removeSelection(this.selectId);
        }
        else {
            if (value == this.multiSelectIsChecked)
                return;
            this.multiSelectIsChecked = value;
            this.datasource.toggleAllFilteredSelected(this.multiSelectIsChecked);
        }
    }
    get isChecked() {
        if (this.selectId)
            return this._isChecked;
        else
            return this.multiSelectIsChecked;
    }
    onSelectedIdsChanges(selectedIds) {
        if (this.selectId) {
            const isInSelection = selectedIds.indexOf(this.selectId) >= 0;
            if (isInSelection !== this._isChecked)
                this._isChecked = isInSelection;
        }
        else
            this.multiSelectIsChecked = this.datasource.dataFiltered.length === selectedIds.length && selectedIds.length > 0;
    }
    onClick($event) {
        $event.stopPropagation();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceSelectionCheckboxComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.13", type: DatasourceSelectionCheckboxComponent, isStandalone: false, selector: "ds-selection-checkbox", inputs: { datasource: "datasource", allowMultiSelect: "allowMultiSelect", selectId: ["ds-select", "selectId"], ariaLabel: ["aria-label", "ariaLabel"], disable: "disable" }, viewQueries: [{ propertyName: "checkboxControl", first: true, predicate: ["checkboxcontrol"], descendants: true, static: true }], ngImport: i0, template: `
        <input [attr.aria-label]="ariaLabel" type="checkbox" #checkboxcontrol [(ngModel)]="isChecked" (click)="onClick($event)" [disabled]="disable" style="transform:scale(1.3)"/>
    `, isInline: true, dependencies: [{ kind: "directive", type: i1$3.CheckboxControlValueAccessor, selector: "input[type=checkbox][formControlName],input[type=checkbox][formControl],input[type=checkbox][ngModel]" }, { kind: "directive", type: i1$3.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i1$3.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceSelectionCheckboxComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'ds-selection-checkbox',
                    template: `
        <input [attr.aria-label]="ariaLabel" type="checkbox" #checkboxcontrol [(ngModel)]="isChecked" (click)="onClick($event)" [disabled]="disable" style="transform:scale(1.3)"/>
    `,
                    standalone: false
                }]
        }], propDecorators: { datasource: [{
                type: Input
            }], allowMultiSelect: [{
                type: Input
            }], selectId: [{
                type: Input,
                args: ['ds-select']
            }], ariaLabel: [{
                type: Input,
                args: ['aria-label']
            }], disable: [{
                type: Input
            }], checkboxControl: [{
                type: ViewChild,
                args: ['checkboxcontrol', { static: true }]
            }] } });

/* eslint-disable @angular-eslint/no-input-rename */
class DatasourceFilterDirective {
    constructor() {
        this.logger = inject(LoggerService);
        this.filterMinLength = 0;
        this.filterOperator = FilterOperator.Contains;
        this.filterInput = inject(ElementRef).nativeElement;
        this.subscriptions = new Array();
        this.oldFilter = undefined;
    }
    set dsfilter(filter) {
        if (!(Array.isArray(filter) || typeof filter === 'string'))
            return;
        this.filterFields = ensureArray(filter);
    }
    get inputWithCheckedProperty() {
        return this.filterInput.type === 'checkbox' || this.filterInput.type === 'radio';
    }
    assertInputWithCheckedProperty() {
        if (!this.inputWithCheckedProperty)
            this.logger.error('DatasourceFilterDirective: filter input has no check property');
    }
    set setDatasource(value) {
        this.datasource = value;
    }
    set filterOperatorAsString(value) {
        this.filterOperator = FilterOperator[value];
    }
    get filterValue() {
        if (this.inputWithCheckedProperty)
            return this.filterTrueValue;
        else
            return this.filterInput.value;
    }
    get isInputChecked() {
        this.assertInputWithCheckedProperty();
        return this.filterInput.checked;
    }
    ngOnInit() {
        this.subscriptions.push(this.datasource.onFilterItems.subscribe(filterItems => this.onFilterChanged(filterItems)));
        this.onFilterChanged(this.datasource.getFilterItems());
        this.initDefaultFilter();
    }
    ngOnDestroy() {
        for (const s of this.subscriptions)
            s.unsubscribe();
    }
    initDefaultFilter() {
        if (this.inputWithCheckedProperty && this.isInputChecked)
            this.defaultFilterValues = this.filterTrueValue;
        if (this.defaultFilterValues !== undefined) {
            if (this.inputWithCheckedProperty)
                this.filterInput.checked = true;
            else
                this.filterInput.value = this.defaultFilterValues;
            this.addFilter();
        }
    }
    onFilterChanged(filterItems) {
        const myFilterItem = filterItems.find((item) => item.fieldnamesAreEqual(this.filterFields) && item.filterOperator == this.filterOperator);
        if (!myFilterItem) {
            this.resetFilter();
            return;
        }
        if (this.filterInput.tagName === 'SELECT') {
            // wenn der Filter aus dem localStorage geladen wird, muss der Filtertext hier verzögert gesetzt werden. vmtl Reihenfolgenproblem beim rendern
            setTimeout(() => this.filterInput.value = myFilterItem.fieldvalues.toString());
            return;
        }
        this.filterInput.value = myFilterItem.fieldvalues.toString();
    }
    resetFilter() {
        this.resetOldValueToEmpty();
        if (this.filterInput.value)
            this.filterInput.value = '';
        if (this.inputWithCheckedProperty && this.filterInput.checked != false)
            this.filterInput.checked = false;
    }
    onBlur() {
        this.addFilter();
    }
    onChange() {
        this.addFilter();
    }
    onInput() {
        this.setOldValueFromDatasource();
        this.timeoutFilter();
    }
    setOldValueFromDatasource() {
        if (this.oldFilter === undefined)
            this.oldFilter = this.datasource.getFirstFilterValueByFieldname(ensureArray(this.filterFields), this.filterOperator);
    }
    resetOldValueToEmpty() {
        if (this.oldFilter)
            this.oldFilter = '';
    }
    /** selbstgebauter debounce */
    timeoutFilter() {
        if (this.timeout != undefined)
            clearTimeout(this.timeout);
        this.timeout = setTimeout(() => {
            this.timeout = undefined;
            this.addFilter();
        }, 800);
    }
    addFilter() {
        if (this.inputWithCheckedProperty) {
            const filterValue = this.isInputChecked ? this.filterValue : '';
            this.datasource.addFilter(this.filterFields, filterValue, this.filterOperator);
            return;
        }
        if (this.filterValue.length >= this.filterMinLength && this.filterValue != this.oldFilter && !this.allEmpty(this.filterValue, this.oldFilter)) {
            this.datasource.addFilter(this.filterFields, this.filterValue, this.filterOperator);
            this.oldFilter = this.filterValue;
        }
    }
    isEmpty(val) {
        return val === undefined || val === null || val === '';
    }
    allEmpty(...args) {
        return args.every(arg => this.isEmpty(arg));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceFilterDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.13", type: DatasourceFilterDirective, isStandalone: false, selector: "[ds-filter]", inputs: { dsfilter: ["ds-filter", "dsfilter"], filterMinLength: ["ds-filter-minlength", "filterMinLength"], setDatasource: ["datasource", "setDatasource"], filterOperatorAsString: ["ds-filter-operator", "filterOperatorAsString"], filterTrueValue: ["ds-filter-true-value", "filterTrueValue"], defaultFilterValues: ["ds-filter-default", "defaultFilterValues"] }, host: { listeners: { "blur": "onBlur()", "change": "onChange()", "input": "onInput()" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceFilterDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ds-filter]',
                    standalone: false
                }]
        }], propDecorators: { dsfilter: [{
                type: Input,
                args: ['ds-filter']
            }], filterMinLength: [{
                type: Input,
                args: ['ds-filter-minlength']
            }], setDatasource: [{
                type: Input,
                args: ['datasource']
            }], filterOperatorAsString: [{
                type: Input,
                args: ['ds-filter-operator']
            }], filterTrueValue: [{
                type: Input,
                args: ['ds-filter-true-value']
            }], defaultFilterValues: [{
                type: Input,
                args: ['ds-filter-default']
            }], onBlur: [{
                type: HostListener,
                args: ['blur']
            }], onChange: [{
                type: HostListener,
                args: ['change']
            }], onInput: [{
                type: HostListener,
                args: ['input']
            }] } });

class DatasourceFilterdisplayComponent {
    constructor() {
        this.subscriptions = new Array();
        this.isFilterSet = false;
        this.filterFieldsVisible = true;
    }
    ngOnInit() {
        this.subscriptions.push(this.datasource.onFilterItems.subscribe((filterItems) => { this.onDatasourceFilterChanged(filterItems); }));
        this.onDatasourceFilterChanged(this.datasource.getFilterItems());
    }
    ngOnDestroy() {
        for (const s of this.subscriptions)
            s.unsubscribe();
    }
    onDatasourceFilterChanged(filterItems) {
        //setTimeout: fix zur verhinderung des fehlers 'Expression changed after checked'
        setTimeout(() => {
            this.isFilterSetCheck(filterItems);
            this.createFilterText(filterItems);
        }, 0);
    }
    isFilterSetCheck(filterItems) {
        this.isFilterSet = filterItems.length > 0 ? true : false;
    }
    createFilterText(filterItems) {
        this.filterText = '';
        if (this.filterFieldsVisible) {
            const textParts = new Array();
            for (const f of filterItems) {
                const fieldvalue = f.hasFieldValue ? f.fieldvalues.join(' OR ') : 'NULL';
                const fieldnamePartText = f.fieldnames.join(' OR ');
                const textpart = `[${fieldnamePartText}] ${FilterOperator[f.filterOperator]} '${fieldvalue}'`;
                textParts.push(textpart);
            }
            this.filterText = textParts.join(' AND ');
        }
        else
            this.filterText = $localize `:@@Datasource.Filter.ResetToDefault.act.Label:Filter zurücksetzen`;
        //optional: Set Filter text for locale (translation);
    }
    clearFilter() {
        this.datasource.clearFilter();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceFilterdisplayComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.13", type: DatasourceFilterdisplayComponent, isStandalone: false, selector: "ds-filterdisplay", inputs: { datasource: "datasource", filterFieldsVisible: "filterFieldsVisible" }, ngImport: i0, template: ` <code class="white-space-normal" *ngIf="isFilterSet && filterFieldsVisible" [title]="filterText">
                    {{filterText}}
                    <i class="fa fa-undo clear-icon" style="cursor:pointer" title="Filter zurücksetzen" (click)="clearFilter()" [shortcut]="'ctrl + backspace'"></i>
                </code>
                <button class="btn btn-default btn-xs white-space-normal" *ngIf="isFilterSet && !filterFieldsVisible" type="button" (click)="clearFilter()" title="Filter zurücksetzen" [shortcut]="'ctrl + backspace'">
                    {{filterText}}
                    <i class="fa fa-times-circle"></i>
                </button>`, isInline: true, styles: ["code{background-color:#d5e0f4;color:#204077;padding:2px 25px 2px 4px;white-space:nowrap!important;overflow:hidden;text-overflow:ellipsis;display:inline-block;max-width:100%;position:relative}.clear-icon{position:absolute;right:5px;top:5px}.white-space-normal{white-space:normal}\n"], dependencies: [{ kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.ShortcutDirective, selector: "[shortcut]", inputs: ["shortcut"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceFilterdisplayComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ds-filterdisplay', template: ` <code class="white-space-normal" *ngIf="isFilterSet && filterFieldsVisible" [title]="filterText">
                    {{filterText}}
                    <i class="fa fa-undo clear-icon" style="cursor:pointer" title="Filter zurücksetzen" (click)="clearFilter()" [shortcut]="'ctrl + backspace'"></i>
                </code>
                <button class="btn btn-default btn-xs white-space-normal" *ngIf="isFilterSet && !filterFieldsVisible" type="button" (click)="clearFilter()" title="Filter zurücksetzen" [shortcut]="'ctrl + backspace'">
                    {{filterText}}
                    <i class="fa fa-times-circle"></i>
                </button>`, standalone: false, styles: ["code{background-color:#d5e0f4;color:#204077;padding:2px 25px 2px 4px;white-space:nowrap!important;overflow:hidden;text-overflow:ellipsis;display:inline-block;max-width:100%;position:relative}.clear-icon{position:absolute;right:5px;top:5px}.white-space-normal{white-space:normal}\n"] }]
        }], propDecorators: { datasource: [{
                type: Input
            }], filterFieldsVisible: [{
                type: Input
            }] } });

class DatasourcePaginationInfoComponent {
    constructor(loggerService, i18n) {
        this.loggerService = loggerService;
        this.i18n = i18n;
        this.variant = 'default';
        this.subscriptions = [];
        this.itemNumberFrom = 0;
        this.itemNumberTo = 0;
        this.itemNumberTotal = 0;
    }
    set setDatasource(value) {
        this.datasource = value;
        this.subscriptions.push(this.datasource.onPaginationChanged.subscribe(paginationInfo => this.paginationChanged(paginationInfo)));
        this.paginationChanged(this.datasource.paginationInfo);
    }
    ngOnDestroy() {
        this.subscriptions.forEach(s => s.unsubscribe());
    }
    paginationChanged(paginationInfo) {
        if (!paginationInfo) {
            this.loggerService.warn('paginationInfo is null');
            return;
        }
        this.itemNumberTotal = paginationInfo.totalNumOfItems;
        const usedPageIndex = paginationInfo.currentPageIndex <= paginationInfo.maxPageIndex ? paginationInfo.currentPageIndex : paginationInfo.maxPageIndex;
        this.itemNumberFrom = (usedPageIndex * paginationInfo.pageSize) + 1;
        this.itemNumberTo = paginationInfo.pageSize > 0 ? this.itemNumberFrom + paginationInfo.currentPageNumOfItems - 1 : this.itemNumberTotal;
    }
    getDataFromToDisplayText(from, to, total) {
        return this.i18n.replace($localize `:@@Datasource.VisibleRecordsRange.dsp.Text:Anzeige $0 bis $1 von $2 Datensätzen`, [from.toString(), to.toString(), total.toString()]);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourcePaginationInfoComponent, deps: [{ token: i1$2.LoggerService }, { token: i2$1.I18nService }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.13", type: DatasourcePaginationInfoComponent, isStandalone: false, selector: "ds-pagination-info", inputs: { setDatasource: ["datasource", "setDatasource"], variant: "variant" }, ngImport: i0, template: `
        <div *ngIf="itemNumberTotal > 0" class="dataTables_info" role="status" aria-live="polite">
            <ng-container *ngIf="!variant || variant==='default'">
                {{ getDataFromToDisplayText(itemNumberFrom, itemNumberTo, itemNumberTotal) }}
            </ng-container>
            <ng-container *ngIf="variant === 'compact'">
                        {{itemNumberFrom}}-{{itemNumberTo}} <span title="Gesamtanzahl Datensätze" i18n-title="@@Datasource.VisibleRecordsRange.dsp.Tooltip">({{itemNumberTotal}})</span>
            </ng-container>
        </div>

        <div *ngIf="itemNumberTotal === 0" class="dataTables_info" role="status" aria-live="polite"></div>
    `, isInline: true, dependencies: [{ kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourcePaginationInfoComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'ds-pagination-info',
                    template: `
        <div *ngIf="itemNumberTotal > 0" class="dataTables_info" role="status" aria-live="polite">
            <ng-container *ngIf="!variant || variant==='default'">
                {{ getDataFromToDisplayText(itemNumberFrom, itemNumberTo, itemNumberTotal) }}
            </ng-container>
            <ng-container *ngIf="variant === 'compact'">
                        {{itemNumberFrom}}-{{itemNumberTo}} <span title="Gesamtanzahl Datensätze" i18n-title="@@Datasource.VisibleRecordsRange.dsp.Tooltip">({{itemNumberTotal}})</span>
            </ng-container>
        </div>

        <div *ngIf="itemNumberTotal === 0" class="dataTables_info" role="status" aria-live="polite"></div>
    `,
                    standalone: false
                }]
        }], ctorParameters: () => [{ type: i1$2.LoggerService }, { type: i2$1.I18nService }], propDecorators: { setDatasource: [{
                type: Input,
                args: ['datasource']
            }], variant: [{
                type: Input
            }] } });

class DatasourcePaginationSizeSelectorComponent {
    constructor(config) {
        this.subscriptions = new Array();
        this.pageSizes = [];
        this.datasourceDataCount = 0;
        this.pageSize = config.pageSize;
    }
    ngOnInit() {
        this.datasource.removeRefreshBlocker();
    }
    ngOnDestroy() {
        this.subscriptions.forEach(s => s.unsubscribe());
    }
    set _pageSizes(values) {
        this.pageSizes = values;
    }
    set _pageSize(value) {
        this.setPageSize(value);
    }
    set setDatasource(datasource) {
        this.datasource = datasource;
        datasource.addRefreshBlocker();
        this.subscriptions.push(this.datasource.onAfterRefresh.subscribe(() => this.datasourceDataCount = this.datasource.data.length), this.datasource.onPaginationChanged.subscribe(paginationInfo => this.setPageSize(paginationInfo.pageSize)));
        this.setPageSize();
    }
    setPageSize(value) {
        if (value !== undefined)
            this.pageSize = value;
        if (this.datasource) {
            const setValue = this.pageSize > 0 ? this.pageSize : 999999999;
            this.datasource.setPageSize(setValue);
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourcePaginationSizeSelectorComponent, deps: [{ token: i1$1.DsDatasourceDefaultConfig }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.13", type: DatasourcePaginationSizeSelectorComponent, isStandalone: false, selector: "ds-pagination-size-selector", inputs: { _pageSizes: ["pageSizes", "_pageSizes"], _pageSize: ["pageSize", "_pageSize"], setDatasource: ["datasource", "setDatasource"] }, ngImport: i0, template: `
        <div class="dataTables_paginate">
            <ul class="pagination" *ngIf="datasourceDataCount>0">
                <li *ngFor='let itemSize of pageSizes' class="paginate_button noselect page-item"
                    [ngClass]="itemSize === pageSize ? 'active disabled' : ''">
                    <span *ngIf="itemSize > 0" href="#" class="page-link" [ngClass]="itemSize === pageSize ? 'active noselect' : ''"
                        (click)="setPageSize(itemSize)">{{ itemSize }}</span>
                    <span *ngIf="itemSize === 0" href="#" class="page-link" [ngClass]="itemSize === pageSize ? 'active noselect' : ''"
                        (click)="setPageSize(itemSize)" i18n="@@Datasource.Pagination.All.dsp.Label">Alle</span>
                </li>
            </ul>
        </div>`, isInline: true, styles: [".noselect{-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;cursor:pointer}\n"], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourcePaginationSizeSelectorComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ds-pagination-size-selector', template: `
        <div class="dataTables_paginate">
            <ul class="pagination" *ngIf="datasourceDataCount>0">
                <li *ngFor='let itemSize of pageSizes' class="paginate_button noselect page-item"
                    [ngClass]="itemSize === pageSize ? 'active disabled' : ''">
                    <span *ngIf="itemSize > 0" href="#" class="page-link" [ngClass]="itemSize === pageSize ? 'active noselect' : ''"
                        (click)="setPageSize(itemSize)">{{ itemSize }}</span>
                    <span *ngIf="itemSize === 0" href="#" class="page-link" [ngClass]="itemSize === pageSize ? 'active noselect' : ''"
                        (click)="setPageSize(itemSize)" i18n="@@Datasource.Pagination.All.dsp.Label">Alle</span>
                </li>
            </ul>
        </div>`, standalone: false, styles: [".noselect{-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;cursor:pointer}\n"] }]
        }], ctorParameters: () => [{ type: i1$1.DsDatasourceDefaultConfig }], propDecorators: { _pageSizes: [{
                type: Input,
                args: ['pageSizes']
            }], _pageSize: [{
                type: Input,
                args: ['pageSize']
            }], setDatasource: [{
                type: Input,
                args: ['datasource']
            }] } });

class DatasourcePaginationComponent {
    constructor() {
        this.subscriptions = [];
        this.pageItems = [];
        this.pageSize = 0;
        this.totalNumOfItems = 0;
        /** Da this.datasource.currentPageIndex 0-basiert ist, speichert diese Variable die tatsächliche Seitenzahl (currentPageIndex + 1) */
        this.selectedPage = 1;
        this.hasPrevious = false;
        this.hasNext = false;
    }
    ngOnDestroy() {
        this.subscriptions.forEach(s => s.unsubscribe());
    }
    set setDatasource(datasource) {
        this.datasource = datasource;
        this.subscriptions.push(this.datasource.onPaginationChanged.subscribe((paginationInfo) => {
            this.paginationChanged(paginationInfo);
        }));
        this.paginationChanged(this.datasource.paginationInfo);
    }
    previousPage() {
        if (this.hasPrevious) {
            this.datasource.gotoPage(this.datasource.currentPageIndex - 1);
            this.selectedPage--;
        }
    }
    nextPage() {
        if (this.hasNext) {
            this.datasource.gotoPage(this.datasource.currentPageIndex + 1);
            this.selectedPage++;
        }
    }
    updateSelectedPage(pageNumber) {
        this.datasource.gotoPage(pageNumber - 1);
        this.selectedPage = pageNumber;
    }
    gotoPage(event, pageIndex) {
        event.stopPropagation(); //aus unklaren gründen hier nötig, damit im dropdowncontainer der dropdwon sich nicjht schliesst bei klick auf pagination; bei debug hat dort das parant-li dieses spans hier keinen parent. betrifft nur dieses li.
        this.datasource.gotoPage(pageIndex);
    }
    paginationChanged(paginationInfo) {
        this.pageItems = [];
        if (paginationInfo) {
            this.totalNumOfItems = paginationInfo.totalNumOfItems;
            this.pageSize = paginationInfo.pageSize;
            let activePageIndex = paginationInfo.currentPageIndex;
            if (activePageIndex > this.datasource.maxPageIndex)
                activePageIndex = this.datasource.maxPageIndex;
            const minPageItemsIndex = Math.max(0, activePageIndex - 3);
            const maxPageItemsIndex = Math.min(activePageIndex + 3, this.datasource.maxPageIndex);
            for (let pi = minPageItemsIndex; pi <= maxPageItemsIndex; pi++) {
                this.pageItems.push({
                    text: (pi + 1).toString(),
                    pageIndex: pi,
                    isCurrent: pi === activePageIndex
                });
            }
            this.hasPrevious = (activePageIndex > 0);
            this.hasNext = (activePageIndex < this.datasource.maxPageIndex);
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourcePaginationComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.13", type: DatasourcePaginationComponent, isStandalone: false, selector: "ds-pagination", inputs: { setDatasource: ["datasource", "setDatasource"], variant: "variant" }, ngImport: i0, template: `
        <div *ngIf="variant !== 'pageOnly'; else pageSelector" class="dataTables_paginate">
            <ul class="pagination" *ngIf="pageItems.length>0 && totalNumOfItems>0">
                <li class="paginate_button page-item noselect" [ngClass]="!hasPrevious?'disabled':''">
                    <span class="noselect page-link" (click)="previousPage()">‹</span>
                </li>
                <li *ngFor='let page of pageItems' class="paginate_button page-item noselect" [ngClass]="page.isCurrent?'active disabled':''">
                    <span href="#" class="noselect page-link" (click)="gotoPage($event,page.pageIndex)">{{page.text}}</span>
                </li>
                <li class="paginate_button page-item noselect" [ngClass]="!hasNext?'disabled':''">
                    <span href="#" class="noselect page-link" (click)="nextPage()">›</span>
                </li>
            </ul>
        </div>
        <ng-template #pageSelector>
            <fieldset class="page-selector">
                <label for="page" i18n="@@Datasource.Pagination.Page.dsp.Label">Seite</label>
                <button class="btn btn-xs paginate_button" i18n-title="@@Datasource.Pagination.Previous.dsp.Tooltip" title="Vorherige Seite" (click)="previousPage()">
                    <i class="fas fa-chevron-left"></i>
                </button>
                <input id="page" type="number" min="1" (change)="updateSelectedPage($any($event.target).value)" [value]="this.selectedPage">
                <button class="btn btn-xs paginate_button" i18n-title="@@Datasource.Pagination.Next.dsp.Tooltip" title="Nächste Seite" (click)="nextPage()">
                    <i class="fas fa-chevron-right"></i>
                </button>
            </fieldset>
        </ng-template>`, isInline: true, styles: [".noselect{-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;cursor:pointer}.page-selector{margin:.2em 0;display:flex;align-items:center}.page-selector>label{margin:0 1.5em 0 0}.page-selector>input{outline:none;margin:0 .2em;appearance:none;-moz-appearance:textfield;text-align:center;width:5em}.page-selector>input::-webkit-outer-spin-button,input::-webkit-inner-spin-button{-webkit-appearance:none;margin:0}\n"], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourcePaginationComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ds-pagination', template: `
        <div *ngIf="variant !== 'pageOnly'; else pageSelector" class="dataTables_paginate">
            <ul class="pagination" *ngIf="pageItems.length>0 && totalNumOfItems>0">
                <li class="paginate_button page-item noselect" [ngClass]="!hasPrevious?'disabled':''">
                    <span class="noselect page-link" (click)="previousPage()">‹</span>
                </li>
                <li *ngFor='let page of pageItems' class="paginate_button page-item noselect" [ngClass]="page.isCurrent?'active disabled':''">
                    <span href="#" class="noselect page-link" (click)="gotoPage($event,page.pageIndex)">{{page.text}}</span>
                </li>
                <li class="paginate_button page-item noselect" [ngClass]="!hasNext?'disabled':''">
                    <span href="#" class="noselect page-link" (click)="nextPage()">›</span>
                </li>
            </ul>
        </div>
        <ng-template #pageSelector>
            <fieldset class="page-selector">
                <label for="page" i18n="@@Datasource.Pagination.Page.dsp.Label">Seite</label>
                <button class="btn btn-xs paginate_button" i18n-title="@@Datasource.Pagination.Previous.dsp.Tooltip" title="Vorherige Seite" (click)="previousPage()">
                    <i class="fas fa-chevron-left"></i>
                </button>
                <input id="page" type="number" min="1" (change)="updateSelectedPage($any($event.target).value)" [value]="this.selectedPage">
                <button class="btn btn-xs paginate_button" i18n-title="@@Datasource.Pagination.Next.dsp.Tooltip" title="Nächste Seite" (click)="nextPage()">
                    <i class="fas fa-chevron-right"></i>
                </button>
            </fieldset>
        </ng-template>`, standalone: false, styles: [".noselect{-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;cursor:pointer}.page-selector{margin:.2em 0;display:flex;align-items:center}.page-selector>label{margin:0 1.5em 0 0}.page-selector>input{outline:none;margin:0 .2em;appearance:none;-moz-appearance:textfield;text-align:center;width:5em}.page-selector>input::-webkit-outer-spin-button,input::-webkit-inner-spin-button{-webkit-appearance:none;margin:0}\n"] }]
        }], propDecorators: { setDatasource: [{
                type: Input,
                args: ['datasource']
            }], variant: [{
                type: Input
            }] } });
class PageItem {
}

class DatasourceSortDirective {
    constructor(el) {
        // eslint-disable-next-line @angular-eslint/no-input-rename
        this.setIconLeft = false;
        this.subscriptions = new Array();
        this.nativeElement = el.nativeElement;
    }
    ngOnInit() {
        if (this.sortField) {
            this.setClassNone();
            this.subscriptions.push(this.datasource.onSortItems.subscribe((sortItems) => {
                this.onTableSortingChanged(sortItems);
            }));
            this.onTableSortingChanged(this.datasource.getSortItems());
        }
        if (this.sortIconAlign === 'left')
            this.setClassIconLeft();
    }
    ngOnDestroy() {
        for (const s of this.subscriptions)
            s.unsubscribe();
    }
    onTableSortingChanged(sortItems) {
        const mySortItem = sortItems.find(item => item.fieldname === this.sortField);
        if (!mySortItem)
            this.setClassNone();
        else {
            if (mySortItem.reverse)
                this.setClassDesc();
            else
                this.setClassAsc();
        }
    }
    setClassNone() {
        this.removeClass('sorting_asc');
        this.removeClass('sorting_desc');
        this.addClass('sorting');
    }
    setClassAsc() {
        this.addClass('sorting_asc');
        this.removeClass('sorting_desc');
        this.removeClass('sorting');
    }
    setClassDesc() {
        this.addClass('sorting_desc');
        this.removeClass('sorting_asc');
        this.removeClass('sorting');
    }
    setClassIconLeft() {
        this.addClass('iconleft');
    }
    removeClass(className) {
        this.nativeElement.classList.remove(className);
    }
    addClass(className) {
        if (!this.nativeElement.classList.contains(className))
            this.nativeElement.classList.add(className);
    }
    onClick(event) {
        if (!this.sortField)
            return;
        if (this.nativeElement.classList.contains('sorting_asc'))
            this.datasource.sortDesc(this.sortField, event.ctrlKey);
        else
            this.datasource.sortAsc(this.sortField, event.ctrlKey);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceSortDirective, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.13", type: DatasourceSortDirective, isStandalone: false, selector: "[ds-sort]", inputs: { sortField: ["ds-sort", "sortField"], datasource: "datasource", setIconLeft: ["icon-left", "setIconLeft"], sortIconAlign: ["ds-sort-icon-align", "sortIconAlign"] }, host: { listeners: { "click": "onClick($event)" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceSortDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ds-sort]',
                    standalone: false
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }], propDecorators: { sortField: [{
                type: Input,
                args: ['ds-sort']
            }], datasource: [{
                type: Input
            }], setIconLeft: [{
                type: Input,
                args: ['icon-left']
            }], sortIconAlign: [{
                type: Input,
                args: ['ds-sort-icon-align']
            }], onClick: [{
                type: HostListener,
                args: ['click', ['$event']]
            }] } });

class DatasourceModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.2.13", ngImport: i0, type: DatasourceModule, declarations: [DatasourceComponent,
            DatasourceFilterDirective,
            DatasourceFilterdisplayComponent,
            DatasourceSortDirective,
            DatasourcePaginationInfoComponent,
            DatasourcePaginationComponent,
            DatasourceSelectionCheckboxComponent,
            DatasourcePaginationSizeSelectorComponent], imports: [CommonModule, FormsModule,
            GlobalKeyboardShortcutsModule], exports: [DatasourceComponent,
            DatasourceFilterDirective,
            DatasourceFilterdisplayComponent,
            DatasourceSortDirective,
            DatasourcePaginationInfoComponent,
            DatasourcePaginationComponent,
            DatasourceSelectionCheckboxComponent,
            DatasourcePaginationSizeSelectorComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceModule, providers: [DecimalPipe], imports: [CommonModule, FormsModule,
            GlobalKeyboardShortcutsModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: DatasourceModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule, FormsModule,
                        GlobalKeyboardShortcutsModule
                    ],
                    declarations: [
                        DatasourceComponent,
                        DatasourceFilterDirective,
                        DatasourceFilterdisplayComponent,
                        DatasourceSortDirective,
                        DatasourcePaginationInfoComponent,
                        DatasourcePaginationComponent,
                        DatasourceSelectionCheckboxComponent,
                        DatasourcePaginationSizeSelectorComponent
                    ],
                    exports: [
                        DatasourceComponent,
                        DatasourceFilterDirective,
                        DatasourceFilterdisplayComponent,
                        DatasourceSortDirective,
                        DatasourcePaginationInfoComponent,
                        DatasourcePaginationComponent,
                        DatasourceSelectionCheckboxComponent,
                        DatasourcePaginationSizeSelectorComponent
                    ],
                    providers: [DecimalPipe],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { AggregateParam, DatasourceCSVService, DatasourceComponent, DatasourceFilterDirective, DatasourceFilterdisplayComponent, DatasourceModule, DatasourcePaginationComponent, DatasourcePaginationInfoComponent, DatasourcePaginationSizeSelectorComponent, DatasourceSelectionCheckboxComponent, DatasourceSortDirective, FilterItem, FilterOperator, IFilterItem, ISortItem, PageItem, PaginationInfo, RefreshParams, RefreshResult, RefreshType, SortItem, aggregateFunctionImplementations };
//# sourceMappingURL=sowatech-shared-ds-datasource.mjs.map
