import exifr from 'exifr';
import * as i0 from '@angular/core';
import { Injectable } from '@angular/core';
import * as i1 from '@sowatech/shared/logger';

class ImageTransformBuilder {
    constructor(imageType, orientation, canvas) {
        this.imageType = imageType;
        this.imageOrientation = orientation;
        this.canvas = canvas;
    }
    static async loadFromSrcStringAsync(base64SrcString) {
        const imgType = `image/${base64SrcString.match(/(data:image\/)(jpe?g|png)/i)[2]}`;
        if (!base64SrcString.startsWith('data'))
            base64SrcString = 'data:image/jpeg;base64,' + base64SrcString;
        const orientation = await ImageTransformBuilder.extractOrientationAsync(base64SrcString);
        const canvas = await this.initCanvasAsync(base64SrcString);
        return new ImageTransformBuilder(imgType, orientation, canvas);
    }
    static loadFromFileAsync(file) {
        const reader = new FileReader();
        return new Promise((resolve, reject) => {
            reader.onload = (e) => resolve(ImageTransformBuilder.loadFromSrcStringAsync(e.target.result));
            reader.onerror = (e) => reject(e.target.error);
            reader.readAsDataURL(file);
        });
    }
    toBase64(quality) {
        return this.canvas.toDataURL(this.imageType, quality);
    }
    copyInstance() {
        const canvasCopy = document.createElement('canvas');
        canvasCopy.width = this.canvas.width;
        canvasCopy.height = this.canvas.height;
        canvasCopy.getContext('2d').drawImage(this.canvas, 0, 0);
        return new ImageTransformBuilder(this.imageType, this.imageOrientation, canvasCopy);
    }
    fixOrientation() {
        //http://jpegclub.org/exif_orientation.html
        switch (this.imageOrientation) {
            case 1:
            case 2:
            case 4:
            case 5:
            case 7:
                return; //unchanged
            case 3:
                this.rotate(180);
                break;
            case 6:
                this.rotate(90);
                break;
            case 8:
                this.rotate(270);
                break;
            default:
                throw new Error('Unknown orientation => ' + this.imageOrientation);
        }
        return this;
    }
    shrinkTo(newWidth, newHeight) {
        if (this.canvas.width >= this.canvas.height) {
            newWidth = Math.min(this.canvas.width, newWidth);
            newHeight = Math.min(this.canvas.width, newWidth);
        }
        else {
            newWidth = Math.min(this.canvas.height, newHeight);
            newHeight = Math.min(this.canvas.height, newHeight);
        }
        const shrinked = document.createElement('canvas');
        if (this.canvas.width >= this.canvas.height) {
            const scaleFactorWidth = this.canvas.width > 0 ? newWidth / this.canvas.width : 0;
            shrinked.width = newWidth;
            shrinked.height = this.canvas.height * scaleFactorWidth;
        }
        else {
            const scaleFactorHeight = this.canvas.height > 0 ? newHeight / this.canvas.height : 0;
            shrinked.width = this.canvas.width * scaleFactorHeight;
            shrinked.height = newHeight;
        }
        shrinked.getContext('2d').drawImage(this.canvas, 0, 0, shrinked.width, shrinked.height);
        this.canvas = shrinked;
        return this;
    }
    crop(cropType, cropCoords) {
        if (cropType === 'square') {
            const isLandscape = this.canvas.width > this.canvas.height;
            const minImageSize = Math.min(this.canvas.width, this.canvas.height);
            cropCoords.left = isLandscape ? (this.canvas.width / 2) - (minImageSize / 2) : 0;
            cropCoords.top = isLandscape ? 0 : (this.canvas.height / 2) - (minImageSize / 2);
            cropCoords.width = minImageSize;
            cropCoords.height = minImageSize;
        }
        else if (cropType === 'coords') {
            cropCoords.left ??= (this.canvas.width / 2) - (cropCoords.width / 2);
            cropCoords.top ??= (this.canvas.height / 2) - (cropCoords.height / 2);
            cropCoords.width ??= 0;
            cropCoords.height ??= 0;
        }
        const imageData = this.canvas.getContext('2d').getImageData(cropCoords.top, cropCoords.left, cropCoords.width, cropCoords.height);
        const cropped = document.createElement('canvas');
        cropped.width = imageData.width;
        cropped.height = imageData.height;
        cropped.getContext('2d').putImageData(imageData, 0, 0);
        this.canvas = cropped;
        return this;
    }
    setMarker(options) {
        options.markercolor ??= 'red';
        const defaultMarkerType = 'rect';
        options.markertype ??= defaultMarkerType;
        const context = this.canvas.getContext('2d');
        context.strokeStyle = options.markercolor;
        if (options.markertype === defaultMarkerType)
            context.strokeRect(options.left, options.top, options.width, options.height);
        else {
            context.beginPath();
            const half = options.width / 2;
            context.lineWidth = options.lineWidth;
            context.arc(options.left + half, options.top + half, half, 0, 2 * Math.PI);
            context.stroke();
        }
        return this;
    }
    setWatermark(text, fontSize) {
        const context = this.canvas.getContext('2d');
        context.save();
        context.font = `${fontSize ?? 46}pt Arial`;
        context.fillStyle = 'white';
        context.shadowColor = 'grey';
        context.shadowOffsetX = 1;
        context.shadowOffsetY = 1;
        context.shadowBlur = 10;
        context.textBaseline = 'alphabetic';
        context.fillText(text, 20, this.canvas.height - 20);
        context.restore();
        return this;
    }
    rotate(angleInDegrees) {
        const rotated = document.createElement('canvas');
        rotated.width = angleInDegrees === 180 ? this.canvas.width : this.canvas.height;
        rotated.height = angleInDegrees === 180 ? this.canvas.height : this.canvas.width;
        const rotatedContext = rotated.getContext('2d');
        rotatedContext.save();
        rotatedContext.translate(rotated.width / 2, rotated.height / 2);
        rotatedContext.rotate(angleInDegrees * Math.PI / 180);
        rotatedContext.drawImage(this.canvas, -this.canvas.width / 2, -this.canvas.height / 2);
        rotatedContext.restore();
        this.canvas = rotated;
    }
    static createImageAsync(base64) {
        return new Promise((resolve, reject) => {
            const img = new Image();
            img.onload = () => resolve(img);
            img.onerror = (e) => reject(e);
            img.src = base64;
        });
    }
    static async initCanvasAsync(base64) {
        const img = await this.createImageAsync(base64);
        const canvas = document.createElement('canvas');
        canvas.width = img.width;
        canvas.height = img.height;
        canvas.getContext('2d').drawImage(img, 0, 0, img.width, img.height);
        return canvas;
    }
    static async extractOrientationAsync(imageData) {
        let exifOrientation;
        try {
            exifOrientation = await exifr.orientation(imageData);
        }
        catch (error) {
            exifOrientation = 1;
        }
        exifOrientation = parseInt((exifOrientation ?? 1).toString(), 10);
        return exifOrientation;
    }
}

/**
 * @deprecated Use ImageTransformBuilder instead
 */
class ImageTransformService {
    constructor(logger) {
        this.logger = logger;
    }
    getBase64FromFile(file) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.onload = (e) => resolve(e.target.result);
            reader.onerror = (e) => reject(e);
            reader.readAsDataURL(file);
        });
    }
    getBase64ImageSrcString(base64) {
        return 'data:image/jpeg;base64,' + base64;
    }
    getBase64FromImageSrcUrl(dataurlBase64) {
        return dataurlBase64.replace(/data:image\/jpeg;base64,/g, '');
    }
    async shrinkImage(base64, options) {
        this.logger.log('ImageTransformService.shrinkImage');
        try {
            const imageBase64 = base64.startsWith('data') ? base64 : this.getBase64ImageSrcString(base64);
            const img = await this.loadImage(imageBase64);
            if (img.width >= img.height) {
                options.width = Math.min(img.width, options.width);
                options.height = Math.min(img.width, options.width);
            }
            else {
                options.width = Math.min(img.height, options.height);
                options.height = Math.min(img.height, options.height);
            }
            const canvas = document.createElement('canvas');
            canvas.width = options.width;
            canvas.height = options.height;
            this.logger.log('ImageTransformService.shrinkImage/img.onload');
            const isLandscape = img.width >= img.height;
            let scaleFactor = 0;
            let width = 0;
            let height = 0;
            if (isLandscape) {
                scaleFactor = img.width > 0 ? canvas.width / img.width : 0;
                width = canvas.width;
                height = img.height * scaleFactor;
                canvas.height = height;
            }
            else {
                scaleFactor = img.height > 0 ? canvas.height / img.height : 0;
                width = img.width * scaleFactor;
                height = canvas.height;
                canvas.width = width;
            }
            const context = canvas.getContext('2d');
            context.drawImage(img, 0, 0, width, height);
            const dataurl = canvas.toDataURL('image/jpeg', options.quality);
            const resultBase64 = this.getBase64FromImageSrcUrl(dataurl);
            return Promise.resolve(resultBase64);
        }
        catch (err) {
            return Promise.reject(err);
        }
    }
    loadImage(base64) {
        return new Promise((resolve, reject) => {
            const img = new Image();
            img.onload = (e) => resolve(img);
            img.onerror = (e) => reject(e);
            img.src = base64;
        });
    }
    async setMarkerOnImage(base64, options) {
        this.logger.log('ImageTransformService.setMarkerOnImage');
        try {
            const img = await this.loadImage(this.getBase64ImageSrcString(base64));
            this.logger.log('ImageTransformService.setMarkerOnImage/img.onload');
            const canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            const context = canvas.getContext('2d');
            context.drawImage(img, 0, 0);
            context.translate(0.5, 0.5);
            const markercolor = options.markercolor ? options.markercolor : 'red';
            context.strokeStyle = markercolor;
            const markertype = options.markertype ? options.markertype : 'rect';
            if (markertype == 'rect')
                context.strokeRect(options.left, options.top, options.width, options.height);
            else {
                context.beginPath();
                const half = options.width / 2;
                context.lineWidth = options.lineWidth;
                context.arc(options.left + half, options.top + half, half, 0, 2 * Math.PI);
                context.stroke();
            }
            const dataurl = canvas.toDataURL('image/jpeg');
            const resultBase64 = dataurl.replace(/data:image\/jpeg;base64,/g, '');
            return Promise.resolve(resultBase64);
        }
        catch (err) {
            return Promise.reject(err);
        }
    }
    async fixOrientation(base64) {
        this.logger.log('ImageTransformService.fixOrientation');
        const exifOrientation = await exifr.orientation(base64);
        const orientation = parseInt((exifOrientation || 1).toString(), 10);
        return this.reOrient(base64, orientation);
    }
    base64ToArrayBuffer(base64) {
        this.logger.log('ImageTransformService.base64ToArrayBuffer');
        const binaryString = window.atob(base64);
        const len = binaryString.length;
        const bytes = new Uint8Array(len);
        for (let i = 0; i < len; i++)
            bytes[i] = binaryString.charCodeAt(i);
        return bytes.buffer;
    }
    reOrient(base64, orientation) {
        this.logger.log('ImageTransformService.reOrient');
        //http://jpegclub.org/exif_orientation.html
        switch (orientation) {
            case 1:
            case 2:
            case 4:
            case 5:
            case 7:
                if (orientation > 1)
                    this.logger.warn('ImageTransformService/reOrient: Unhandled EXIF Orientation=' + orientation.toString());
                return Promise.resolve(base64); //unchanged
            case 3:
                return this.rotateImage(base64, 180);
            case 6:
                return this.rotateImage(base64, 90);
            case 8:
                return this.rotateImage(base64, 270);
            default:
                throw new Error('Unknown orientation => ' + orientation);
        }
    }
    async rotateImage(base64, angleInDegrees) {
        this.logger.log('ImageTransformService.rotateImage');
        try {
            const img = await this.loadImage(this.getBase64ImageSrcString(base64));
            this.logger.log('ImageTransformService.rotateImage/img.onload');
            const canvas = document.createElement('canvas');
            const maxsize = Math.max(img.width, img.height);
            canvas.width = maxsize;
            canvas.height = maxsize;
            const context = canvas.getContext('2d');
            context.save();
            context.translate(canvas.width / 2, canvas.height / 2);
            context.rotate(angleInDegrees * Math.PI / 180);
            context.drawImage(img, -img.width / 2, -img.height / 2);
            context.restore();
            const dataurl = canvas.toDataURL('image/jpeg');
            const resultBase64 = this.getBase64FromImageSrcUrl(dataurl);
            //bei drehung um vielfaches von 90 kann bild wieder sauber geschnitten werden
            if (angleInDegrees == 90 || angleInDegrees == 270 || angleInDegrees == -90 || angleInDegrees == 180) {
                const cropwidth = angleInDegrees == 180 ? img.width : img.height;
                const cropheight = angleInDegrees == 180 ? img.height : img.width;
                const croppedImage = await this.cropImage(resultBase64, { cropCoords: { width: cropwidth, height: cropheight } });
                return Promise.resolve(croppedImage);
            }
            else
                return Promise.resolve(resultBase64);
        }
        catch (err) {
            return Promise.reject(err);
        }
    }
    setWatermark(base64, text, config) {
        this.logger.log('ImageTransformService.setWatermark');
        return new Promise((resolve, reject) => {
            const img = new Image();
            img.src = this.getBase64ImageSrcString(base64);
            img.onload = () => {
                this.logger.log('ImageTransformService.setWatermark/img.onload');
                const canvas = document.createElement('canvas');
                canvas.width = img.width;
                canvas.height = img.height;
                const context = canvas.getContext('2d');
                context.save();
                context.drawImage(img, 0, 0);
                context.font = '46pt Arial';
                context.fillStyle = 'white';
                context.shadowColor = 'grey';
                context.shadowOffsetX = 1;
                context.shadowOffsetY = 1;
                context.shadowBlur = 10;
                context.textBaseline = 'alphabetic';
                context.scale(1, 1);
                context.fillText(text, 20, img.height - 20);
                context.restore();
                const dataurl = canvas.toDataURL('image/jpeg');
                return resolve(this.getBase64FromImageSrcUrl(dataurl));
            };
            img.onerror = (err) => reject(err);
        });
    }
    async cropImage(base64, options) {
        this.logger.log('ImageTransformService.cropImage');
        try {
            const img = await this.loadImage(this.getBase64ImageSrcString(base64));
            this.logger.log('ImageTransformService.cropImage/img.onload');
            const canvas = document.createElement('canvas');
            let left = 0;
            let top = 0;
            let width = 0;
            let height = 0;
            let validOptions = true;
            const cropType = options.cropType || 'coords';
            if (cropType == 'square') {
                const isLandscape = img.width > img.height;
                const minImageSize = Math.min(img.width, img.height);
                left = isLandscape ? img.width / 2 - minImageSize / 2 : 0;
                top = isLandscape ? 0 : img.height / 2 - minImageSize / 2;
                width = minImageSize;
                height = minImageSize;
            }
            if (cropType == 'coords') {
                if (options.cropCoords) {
                    left = options.cropCoords.left || img.width / 2 - options.cropCoords.width / 2;
                    top = options.cropCoords.top || img.height / 2 - options.cropCoords.height / 2;
                    width = options.cropCoords.width;
                    height = options.cropCoords.height;
                }
                else
                    validOptions = false;
            }
            if (cropType != 'square' && cropType != 'coords')
                validOptions = false;
            canvas.width = width;
            canvas.height = height;
            const context = canvas.getContext('2d');
            context.drawImage(img, left, top, width, height, 0, 0, width, height);
            const dataurl = canvas.toDataURL('image/jpeg');
            const resultBase64 = this.getBase64FromImageSrcUrl(dataurl);
            return resultBase64;
        }
        catch (err) {
            return Promise.reject(err);
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: ImageTransformService, deps: [{ token: i1.LoggerService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: ImageTransformService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: ImageTransformService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: () => [{ type: i1.LoggerService }] });

/**
 * Generated bundle index. Do not edit.
 */

export { ImageTransformBuilder, ImageTransformService };
//# sourceMappingURL=sowatech-shared-image-transform.mjs.map
