import * as i0 from '@angular/core';
import { inject, input, effect, untracked, Directive } from '@angular/core';
import { createEffect, ofType, Actions } from '@ngrx/effects';
import { createAction, createSelector, Store } from '@ngrx/store';
import { GenericEditDialogService } from '@sowatech/shared/swt-generic';
import { catchError, of } from 'rxjs';
import { map, filter, tap } from 'rxjs/operators';
import { LoggerService } from '@sowatech/shared/logger';
import { DatasourceComponent, RefreshType } from '@sowatech/shared/ds-datasource';

/**
 * Effect, der als Reaktion auf eine Failure Action den ErrorHandler mit angegebenem ActionText auslöst.
 * ```ts
 * loadListError$ = createErrorHandlerEffect(apiActions.loadListFailure, 'Laden der Liste');
 * ```
 */
function createErrorHandlerEffect(action, actionText, actions$ = inject(Actions)) {
    return createEffect(() => actions$.pipe(ofType(action), map(ac => errorAction(ac.error, { handling: 'messageBox', actionText }))));
}
/**
 * Effect, der als Reaktion auf eine Failure Action den Fehler im Edit Dialog anzeigt.
 * ```ts
 * createError$ = createDialogErrorEffect(apiActions.createFailure);
 * ```
 */
function createDialogErrorEffect(action, actions$ = inject(Actions)) {
    return createEffect(() => actions$.pipe(ofType(action), map(ac => errorAction(ac.error, { handling: 'dialog', actionText: undefined }))));
}
/**
 * props für Failure Actions.
 * ```ts
 * createActionGroup({
 *     name: 'My API',
 *     events: {
 *         'Update Success': emptyProps(),
 *         'Update Failure': errorProps(),
 *     },
 * });
 *
 * catchError(err => of(apiActions.updateFailure(err)))
 * ```
 */
const errorProps = () => (error) => ({ error: { name: error.name, message: error.message, status: error.status, error: error.error } });
/**
 * Abkürzung fürs Mappen zu einer Failure Action.
 *
 * ohne mapFailure: `catchError(err => of(apiActions.updateFailure(err)))`
 *
 * mit mapFailure: `mapFailure(apiActions.updateFailure)`
 */
const mapFailure = (failureAction) => catchError(err => of(failureAction(err)));
/**
 * globale Error Action, über die sämtliches ErrorHandling abgehandelt wird
 */
const errorAction = createAction('[App] Error', (error, errorOptions) => ({ error, actionText: errorOptions.actionText, handling: errorOptions.handling }));
/**
 * Globaler Effect, der von `createDialogErrorEffect` gemappte Fehler im Dialog anzeigt.
 * Der Effect muss im AppModule registriert werden:
 * ```ts
 * EffectsModule.forRoot({ globalDialogErrorHandlingEffect })
 * ```
 */
const globalDialogErrorHandlingEffect = createEffect((actions$ = inject(Actions), dialog = inject(GenericEditDialogService)) => {
    return actions$.pipe(ofType(errorAction), filter(({ handling }) => handling === 'dialog'), map(({ error }) => error.error.titleKey && error.error.detailKey ? `${error.error.titleKey}: ${error.error.detailKey}` : error.message), tap((msg) => dialog.showErrors([msg])));
}, { functional: true, dispatch: false });
/**
 * Globaler Effect, der von `createErrorHandlerEffect` gemappte Fehler über den ErrorHandler anzeigt
 * Ist eine Factory, weil der ErrorHandler nicht im shared liegt.
 * Der Effect muss im AppModule registriert werden:
 * ```ts
 * EffectsModule.forRoot({ globalErrorHandlingEffect: globalErrorHandlingEffectFactory(ErrorHandlerService) })
 * ```
 */
function globalErrorHandlingEffectFactory(errorHandlerType) {
    return createEffect((actions$ = inject(Actions), errorHandler = inject(errorHandlerType)) => {
        return actions$.pipe(ofType(errorAction), filter(({ handling }) => handling === 'messageBox'), tap(({ error, actionText }) => errorHandler.handleError(error, actionText)));
    }, { functional: true, dispatch: false });
}

/**
 * Abkürzung von `createEffect` für den Standardfall, wenn man auf Actions reagiert
 * ```ts
 * loadList$ = createActionEffect(
 *     [listActions.init, apiActions.createSuccess],
 *     switchMap(() => this.webApi.getList()),
 * );
 * create$ = createActionEffect(
 *     listActions.create,
 *     pipe(
 *         showDialog(new CreateDialog()),
 *         switchMap(({ dto }) => this.webApi.create(dto)),
 *     ),
 * );
 * ```
 */
function createActionEffect(actions, fn, actions$ = inject(Actions)) {
    const actionList = Array.isArray(actions) ? actions : [actions];
    return createEffect(() => actions$.pipe(ofType(...actionList), fn));
}
class EffectBase {
    constructor() {
        this.actions$ = inject(Actions);
    }
    on(actions, fn) {
        return createActionEffect(actions, fn, this.actions$);
    }
    onError(action, actionText) {
        return createErrorHandlerEffect(action, actionText, this.actions$);
    }
    onDialogError(action) {
        return createDialogErrorEffect(action, this.actions$);
    }
}

/**
 * Meta Reducer fürs Loggen der Actions. Wird konfiguriert im AppModule:
 * ```ts
 * StoreModule.forRoot({}, { metaReducers: [loggingMetaReducer] }),
 * ```
 */
function loggingMetaReducer(reducer) {
    const logger = inject(LoggerService);
    return function (state, action) {
        if (!action.type.startsWith('@ngrx/'))
            logger.log(action.type);
        return reducer(state, action);
    };
}

class ExternalRefreshAdapter {
    /**
     * Initial State für external Refresh.
     * ```ts
     * type State = ExternalRefreshState<MyEntity> & { loading: boolean };
     * const initialState: State = adapter.getInitialState({ loading: false });
     * ```
     * @param extraState zusätzlicher State, der neben dem `refreshResult` abgelegt wird
     */
    getInitialState(extraState) {
        if (!extraState)
            extraState = {};
        return { ...extraState, refreshResult: null, loading: false };
    }
    /**
     * Zum Updaten des Refresh Results
     * ```ts
     * on(apiActions.loadSuccess, (state, { refreshResult }) => adapter.update(refreshResult, state)),
     * ```
     */
    update(result, state) {
        let refreshResult = result;
        if (state.refreshResult && result.refreshParams.aggregateParam.aggregatesOnly)
            refreshResult = { ...state.refreshResult, aggregateResults: result.aggregateResults };
        return { ...state, refreshResult };
    }
    /**
     * Zum Updaten des Refresh Results über `externalRefreshProps`
     * ```ts
     * // Action
     * 'Load Success': externalRefreshProps<MyEntity>(),
     * // Reducer
     * on(apiActions.loadSuccess, adapter.updateFromAction()),
     * ```
     */
    updateFromAction() {
        return (state, action) => this.update(action.refreshResult, state);
    }
    /**
     * Löscht das Refresh Result
     */
    clear(state) {
        return { ...state, refreshResult: null };
    }
    setLoading(loading, state) {
        return { ...state, loading };
    }
    /**
     * Zum Setzen von `loading`
     * ```ts
     * on(apiActions.loadSuccess, apiAction.loadFailure, adapter.setLoadingStatic(false)),
     * ```
     */
    setLoadingStatic(loading) {
        return (state) => this.setLoading(loading, state);
    }
    /**
     * Setzt Loading auf `true`, wenn nicht nur für die Aggregate Params neu geladen wird.
     * ```ts
     * on(apiActions.load, (state, { refreshParams }) => adapter.startLoading(refreshParams, state)),
     * ```
     */
    startLoading(refreshParams, state) {
        if (refreshParams.aggregateParam.aggregatesOnly)
            return state;
        return this.setLoading(true, state);
    }
    /**
     * Gibt ein paar nützliche Selectors zurück:
     */
    getSelectors() {
        /**
         * Gibt die aktuell geladenen Daten zurück
         * ```ts
         * const selectCurrentPage = createSelector(selectRefreshResult, result => selectList(result));
         * ```
         */
        function selectList(state) {
            if (!state.refreshResult)
                return [];
            return state.refreshResult.data;
        }
        /**
         * Holt ein geladenes Item anhand der ID
         * ```ts
         * const selectFocusedItem createSelector(
         *     selectRefreshResult,
         *     selectFocusedId,
         *     (result, id) => selectItem(result, id)
         * );
         * ```
         */
        function selectItem(state, id) {
            if (!state.refreshResult)
                return undefined;
            const idField = state.refreshResult.refreshParams.idField;
            return selectList(state).find(item => item[idField] === id);
        }
        /**
         * Gibt alle IDs der aktuell geladenen Daten zurück
         * ```ts
         * const selectPageIds = createSelector(selectRefreshResult, result => selectIds(result));
         * ```
         */
        function selectIds(state) {
            if (!state.refreshResult)
                return [];
            const idField = state.refreshResult.refreshParams.idField;
            return selectList(state).map(item => item[idField]);
        }
        return {
            selectList,
            selectItem,
            selectIds,
        };
    }
    /**
     * Da der Store immutable ist, muss der Selector für die Datasource überschrieben werden.
     * (SHANG-465)
     */
    createRefreshResultSelector(selector) {
        return createSelector(selector, result => JSON.parse(JSON.stringify(result)));
    }
}
/**
 * Erstellt einen Adapter für den Umgang mit serverseitig geladenen Collections.
 * ```ts
 * const adapter = createExternalRefreshAdapter<MyEntity>();
 *
 * type State = ExternalRefreshState<MyEntity>;
 *
 * const initialState: State = adapter.getInitialState();
 *
 * const myFeature = createFeature({
 *     name: 'myFeature',
 *     reducer: createReducer(
 *         initialState,
 *         on(apiActions.loadSuccess, adapter.updateFromAction()),
 *         on(actions.load, (state, { refreshParams }) => adapter.startLoading(refreshParams, state)),
 *         on(apiActions.loadSuccess, apiActions.loadFailure, adapter.setLoadingStatic(false)),
 *     ),
 *     extraSelectors(baseSelectors) {
 *         const selectRefreshResult = adapter.createRefreshResultSelector(baseSelectors.selectRefreshResult);
 *         return {
 *             selectRefreshResult,
 *         };
 *     },
 * });
 *
 * // Load Action
 * 'Load': externalRefreshParamsProps(),
 * // API Action
 * 'Load Success': externalRefreshResultProps<MyEntity>(),
 * ```
 */
function createExternalRefreshAdapter() {
    return new ExternalRefreshAdapter();
}
function externalRefreshResultProps() {
    return (refreshResult) => ({ refreshResult });
}
function externalRefreshParamsProps() {
    return (refreshParams) => ({ refreshParams: JSON.parse(JSON.stringify(refreshParams)) });
}

class ExternalRefreshActionDirective {
    constructor() {
        this.datasource = inject(DatasourceComponent);
        this.store = inject(Store);
        this.externalRefreshAction = input.required();
        effect(() => {
            const action = this.externalRefreshAction();
            this.datasource.refreshResultRequested.subscribe(params => this.store.dispatch(action(params)));
            untracked(() => this.datasource.refreshType.set(RefreshType.externalRefreshInput));
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: ExternalRefreshActionDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.13", type: ExternalRefreshActionDirective, isStandalone: true, selector: "ds-datasource[externalRefreshAction][externalRefreshResult]", inputs: { externalRefreshAction: { classPropertyName: "externalRefreshAction", publicName: "externalRefreshAction", isSignal: true, isRequired: true, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: ExternalRefreshActionDirective, decorators: [{
            type: Directive,
            args: [{
                    standalone: true,
                    selector: 'ds-datasource[externalRefreshAction][externalRefreshResult]'
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { EffectBase, ExternalRefreshActionDirective, ExternalRefreshAdapter, createActionEffect, createDialogErrorEffect, createErrorHandlerEffect, createExternalRefreshAdapter, errorAction, errorProps, externalRefreshParamsProps, externalRefreshResultProps, globalDialogErrorHandlingEffect, globalErrorHandlingEffectFactory, loggingMetaReducer, mapFailure };
//# sourceMappingURL=sowatech-shared-ngrx.mjs.map
