import { inject } from '@angular/core';
import { Router } from '@angular/router';
import { MessageBoxService } from '@sowatech/shared/dialogs';
import { GenericEditDialogService } from '@sowatech/shared/swt-generic';
import { pipe, of, switchMap, tap, from } from 'rxjs';
import { catchError, filter, map } from 'rxjs/operators';

/** Um aus einem Parameter, der entweder ein Wert, oder eine Funktion ist, den Wert abzuleiten */
function evaluateValueOrFunction(valueOrFunction, parameters) {
    const condition = (x) => typeof x !== 'function';
    return condition(valueOrFunction) ? valueOrFunction : valueOrFunction(parameters);
}
/** Wert, der im Fehlerfall verwendet werden kann, um Fehler zu erkennen und im Observable Stream zu ignorieren */
const ERROR_VALUE = Object.freeze({ _error: true });
/**
 * Operator, um im Fehlerfall den Edit Dialog zu öffnen und den Fehler dort anzuzeigen.
 * Beispiel:
 * ```ts
 * create$.pipe(
 *     showDialog(new CreateDialog()),
 *     webApiCall(dto => this.webApi.create(dto), catchAndShowDialogErrors()),
 * );
 * ```
 */
function catchAndShowDialogErrors(dialog = inject(GenericEditDialogService)) {
    return pipe(catchError(err => {
        const message = err.error?.detailKey ?? err.error?.detail ?? err.message;
        dialog.showErrors([message]);
        return of(ERROR_VALUE);
    }), filter((v) => v !== ERROR_VALUE));
}
/**
 * Zur Anzeige eines Confirm Dialogs. Texte können als konstanter String oder als Funktion, die jedes Mal evaluiert wird, angegeben werden.
 * Beispiel:
 * ```ts
 * delete$.pipe(
 *     showConfirm(({ name }) => `Möchten Sie ${name} wirklich löschen?`, 'Löschen?'),
 *     switchMap(dto => this.webApi.delete(dto)),
 * );
 * ```
 */
function showConfirm(text, title, messageBox = inject(MessageBoxService)) {
    return switchMap(async (x) => {
        const evaluatedTitle = evaluateValueOrFunction(title, x);
        const evaluatedText = evaluateValueOrFunction(text, x);
        await messageBox.confirmDialog(evaluatedText, evaluatedTitle);
        return x;
    });
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
/**
 * @deprecated Nur für debugging
 */
function consoleWarn(label) {
    if (!label)
        tap(console.warn);
    return tap(input => console.warn(label, input));
}
/**
 * @deprecated Nur für debugging
 */
function consoleCount(label) {
    return tap(() => console.count(label));
}
/**
 * Verkürzt das Mappen aller Items einer Liste.
 *
 * Ohne mapEach: `map(list => list.map(item => new Model(item)))`
 *
 * Mit mapEach: `mapEach(item => new Model(item))`
 */
function mapEach(fn) {
    return map((sources) => sources.map(fn));
}
function notUndefined() {
    return filter((x) => x !== undefined);
}
function notNull() {
    return filter((x) => x !== null);
}
/**
 * Filtert ein Observable von `GridColumnButtonEvent` nach dem ident.
 * @param ident
 */
function filterButtonIdent(ident) {
    return filter(({ buttonIdent }) => buttonIdent === ident);
}
/**
 * Examples:
 * ```ts
 * showDialog(new MyDialog()) // Dialog anzeigen
 * showDialog(dto => new MyDialog(dto.flag)) // Dialog anhand der Daten modifizieren
 * showDialog(new MyDialog(), dto => ({ value: dto.a, text: dto.b })) // Dialog vorausfüllen
 * showDialog(new MyDialog(), null, () => ({ lookups: [{ key: 'datasource', datasource: this.lookupDatasource }] })) // Datasource für Grid
 * ```
 */
function showDialog(config, dto = null, options = null, dialog = inject(GenericEditDialogService)) {
    return switchMap(input => {
        const evaluatedConfig = evaluateValueOrFunction(config, input);
        const optionsResult = options?.(input) ?? { mode: null, lookups: [] };
        return dialog.show(evaluatedConfig, dto?.(input) ?? null, optionsResult.mode, optionsResult.lookups);
    });
}
/**
 * Wrapper für einen WebApi Call, mit dem man Fehler behandeln kann, ohne die äußere Subscription zu beenden.
 * Sollte mit einem Error Handler benutzt werden.
 * ```ts
 * webApiCall(() => this.webApi.getList(), handleError('Laden der Liste'))
 * webApiCall(dto => this.webApi.create(dto), catchAndShowDialogErrors())
 * ```
 */
function webApiCall(fn, ...pipes) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    const _pipe = pipe(...pipes);
    return switchMap((input) => from(fn(input)).pipe(_pipe));
}
function navigateTo(fn, extrasFn, router = inject(Router)) {
    return tap(x => router.navigate(fn(x), extrasFn?.(x)));
}
/**
 * Wenn die Bedingung **nicht** erfüllt ist, wird die angegebene Fehlermeldung im Dialog angezeigt.
 * Beispiel:
 * ```ts
 * pipe(
 *     showDialog(new CreateDialog()),
 *     dialogAssert(({ start, end } => end >= start, 'Das Ende darf nicht vor dem Start liegen'),
 * )
 * ```
 */
function dialogAssert(condition, errorMessage, dialog = inject(GenericEditDialogService)) {
    return filter(x => {
        const result = condition(x);
        if (!result)
            dialog.showErrors([errorMessage]);
        return result;
    });
}

/**
 * Generated bundle index. Do not edit.
 */

export { ERROR_VALUE, catchAndShowDialogErrors, consoleCount, consoleWarn, dialogAssert, filterButtonIdent, mapEach, navigateTo, notNull, notUndefined, showConfirm, showDialog, webApiCall };
//# sourceMappingURL=sowatech-shared-operators.mjs.map
